/*
   Copyright (C) 2009 Omat Holding B.V. <info@omat.nl>
   Copyright (C) 2008 Thomas McGuire <thomas.mcguire@gmx.net>

   This library is free software; you can redistribute it and/or modify it
   under the terms of the GNU Library General Public License as published by
   the Free Software Foundation; either version 2 of the License, or (at your
   option) any later version.

   This library is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
   License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to the
   Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
   02110-1301, USA.
*/

#include "collectiondelegate.h"

#include <QModelIndex>
#include <QPainter>

#include <KColorScheme>
#include <KLocale>

#include <akonadi/entitytreemodel.h>
#include <akonadi/collectionstatistics.h>

using namespace Mailody;
using namespace Akonadi;

CollectionDelegate::CollectionDelegate( EntityTreeView* parent )
        : QStyledItemDelegate( parent ), m_etv( parent )
{
}

void CollectionDelegate::initStyleOption( QStyleOptionViewItem *option,
        const QModelIndex &index ) const
{
    QStyleOptionViewItemV4 *noTextOption =
        qstyleoption_cast<QStyleOptionViewItemV4 *>( option );
    QStyledItemDelegate::initStyleOption( noTextOption, index );
    noTextOption->text.clear();
}

void CollectionDelegate::paint( QPainter * painter,
                                const QStyleOptionViewItem & option,
                                const QModelIndex & index ) const
{
    // First, paint the basic, but without the text. We remove the text
    // in initStyleOption(), which gets called by QStyledItemDelegate::paint().
    QStyledItemDelegate::paint( painter, option, index );

    // Now we retrieve the correct style option by calling intiStyleOption from
    // the superclass.
    QStyleOptionViewItemV4 option4 = option;
    QStyledItemDelegate::initStyleOption( &option4, index );
    QString text = option4.text;

    // Now calculate the rectangle for the text
    QStyle *s = m_etv->style();
    const QWidget *widget = option4.widget;
    QRect textRect = s->subElementRect( QStyle::SE_ItemViewItemText, &option4, widget );

    // When checking if the item is expanded, we need to check that for the first
    // column, as Qt only recogises the index as expanded for the first column
    QModelIndex firstColumn = index.model()->index( index.row(), 0, index.parent() );

    if ( option.state & QStyle::State_Selected ) {
        painter->save();
        painter->setPen( option.palette.highlightedText().color() );
    }

    // Draw the unread count after the folder name (in parenthesis)
    if ( index.column() == 0 ) {

        const EntityTreeModel* model = static_cast<const EntityTreeModel*>( index.model() );
        const Collection col =
            model->data( index, EntityTreeModel::CollectionRole ).value<Collection>();
        const int unreadCount = col.statistics().unreadCount();
        const QString unread = QString( QLatin1String( " (%1)" ) ).arg( unreadCount );

        painter->save();

        if ( unreadCount > 0 ) {
            QFont font = painter->font();
            font.setBold( true );
            painter->setFont( font );
        }

        // Squeeze the folder text if it is to big and calculate the rectangles
        // where the folder text and the unread count will be drawn to
        QString folderName = text;
        QFontMetrics fm( painter->fontMetrics() );
        int unreadWidth = fm.width( unread );
        if ( fm.width( folderName ) + unreadWidth > textRect.width() ) {
            folderName = fm.elidedText( folderName, Qt::ElideRight,
                                        textRect.width() - unreadWidth );
        }
        int folderWidth = fm.width( folderName );
        QRect folderRect = textRect;
        QRect unreadRect = textRect;
        folderRect.setRight( textRect.left() + folderWidth );
        unreadRect.setLeft( folderRect.right() );

        // Draw folder name and unread count
        painter->drawText( folderRect, Qt::AlignLeft, folderName );
        KColorScheme::ColorSet cs = ( option.state & QStyle::State_Selected ) ?
                                    KColorScheme::Selection : KColorScheme::View;

        if ( unreadCount > 0 ) {
            QColor unreadColor = KColorScheme( QPalette::Active, cs ).
                                 foreground( KColorScheme::LinkText ).color();
            painter->setPen( unreadColor );
            painter->drawText( unreadRect, Qt::AlignLeft, unread );
        }

        painter->restore();

        if ( option.state & QStyle::State_Selected ) {
            painter->restore();
        }
        return;
    }

    painter->drawText( textRect, option4.displayAlignment, text );

    if ( option.state & QStyle::State_Selected ) {
        painter->restore();
    }
}
