/*
   Copyright (C) 2006-2009 Omat Holding B.V. <info@omat.nl>
   Copyright (C) 2007 Frode M. Døving <frode@lnix.net>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*/

// Own
#include "composer.h"
#include "attachmentwidget.h"
#include "contacttreemodel.h"
#include "contacttooltipproxymodel.h"
#include "global.h"
#include "imagemodel.h"
#include "sendmessage.h"
#include "recentmodel.h"
#include "recentmanager.h"

// Qt
#include <QApplication>
#include <QDockWidget>
#include <QGridLayout>
#include <QLabel>
#include <QMenu>
#include <QSplitter>
#include <QToolBox>
#include <QToolButton>

// KDEPIM
#include <kabc/addressee.h>
#include <kabc/contactgroup.h>
#include <mailtransport/transportmanager.h>
#include <kpimidentities/identitycombo.h>
#include <kpimidentities/identity.h>
#include <akonadi/changerecorder.h>
#include <akonadi/contact/contacteditordialog.h>
#include <akonadi/contact/contactgroupexpandjob.h>
#include <akonadi/entitytreemodel.h>
#include <akonadi/entitytreeview.h>
#include <akonadi/itemcreatejob.h>
#include <akonadi/itemfetchscope.h>
#include <akonadi/itemdeletejob.h>
#include <akonadi/session.h>

// KDE
#include <KAction>
#include <KActionCollection>
#include <KDirLister>
#include <KDirModel>
#include <KLineEdit>
#include <KMessageBox>
#include <KStandardAction>
#include <KStatusBar>
#include <KToggleAction>
#include <KToolBar>

// Sonnet
#include <sonnet/dictionarycombobox.h>

using namespace Mailody;
using namespace Akonadi;

//---------- Composer --------------------------//
Composer::Composer( QWidget* parent, const Akonadi::Collection& mb )
        : KXmlGuiWindow( parent ),
        m_mailbox( mb ),  m_close( false ), m_dirty( false )
{
    setObjectName( "composer#" );
    setCaption( i18n( "Composer" ) );

    connect( parent, SIGNAL( setupFinished() ), SLOT( slotStatusChanged() ) );

    Akonadi::ItemFetchScope scope;
    scope.fetchFullPayload( true );

    // setup addressbook
    ChangeRecorder *monitor = new ChangeRecorder( this );
    monitor->setItemFetchScope( scope );
    monitor->setCollectionMonitored( Collection::root() );
    monitor->setMimeTypeMonitored( KABC::Addressee::mimeType(), true );
    //monitor->setMimeTypeMonitored( KABC::ContactGroup::mimeType(), true );

    ContactTreeModel* addressbookModel = new ContactTreeModel( monitor, this );
    addressbookModel->setIncludeRootCollection( false );

    ContactToolTipProxyModel* tooltip =
        new ContactToolTipProxyModel( this );
    tooltip->setSourceModel( addressbookModel );

    m_addressbookView = new QTreeView( this );
    m_addressbookView->setModel( tooltip );
    m_addressbookView->setContextMenuPolicy( Qt::CustomContextMenu );

    // signal directly from the view
    connect( m_addressbookView,
             SIGNAL( clicked( const QModelIndex& ) ),
             SLOT( slotAddAddressFromAddressBook( const QModelIndex& ) ) );
    connect( m_addressbookView,
             SIGNAL( customContextMenuRequested( const QPoint & ) ),
             SLOT( slotContextMenuAddressBook() ) );

    QDockWidget* addressDock = new QDockWidget( i18n( "Add&ress Book" ), this );
    addressDock->setWidget( m_addressbookView );
    addressDock->setObjectName( "addressDock" );
    addressDock->setFeatures( QDockWidget::DockWidgetMovable |
                              QDockWidget::DockWidgetClosable );
    addDockWidget( Qt::LeftDockWidgetArea, addressDock );

    // setup file browser.
    KDirModel* dirModel = new KDirModel( this );
    m_filebrowser = new QTreeView( this );
    m_filebrowser->setEditTriggers( QAbstractItemView::NoEditTriggers );
    m_filebrowser->setModel( dirModel );
    m_filebrowser->resizeColumnToContents( KDirModel::ModifiedTime );
    m_filebrowser->hideColumn( KDirModel::Owner );
    m_filebrowser->hideColumn( KDirModel::Group );
    m_filebrowser->hideColumn( KDirModel::Type );
    m_filebrowser->hideColumn( KDirModel::Permissions );

    dirModel->dirLister()->openUrl( KUrl( "~" ) );
    connect( m_filebrowser,
             SIGNAL( doubleClicked( const QModelIndex& ) ),
             SLOT( slotAddFile( const QModelIndex& ) ) );

    // images.
    QSplitter *splitter = new QSplitter( Qt::Vertical, this );

    Session *session = new Session( QByteArray( "ImageViewAplication-" ) + QByteArray::number( qrand() ), this );

    ChangeRecorder *changeRecorder = new ChangeRecorder( this );
    changeRecorder->setSession( session );
    changeRecorder->setItemFetchScope( scope );
    changeRecorder->setCollectionMonitored( Collection::root() );
    changeRecorder->setMimeTypeMonitored( "application/x-vnd.kde.image", true );

    EntityTreeModel *model = new EntityTreeModel( changeRecorder, this );
    model->setItemPopulationStrategy( EntityTreeModel::NoItemPopulation );

    EntityTreeView *lv = new EntityTreeView( splitter );
    lv->setModel( model );

    ImageModel* im = new ImageModel( this );
    im->setFetchScope( scope );

    QListView* lv2 = new QListView( splitter );
    lv2->setViewMode( QListView::IconMode );
    lv2->setModel( im );
    connect( lv2, SIGNAL( clicked( const QModelIndex& ) ),
             SLOT( slotImageClicked( const QModelIndex& ) ) );

    connect( lv, SIGNAL( currentChanged( const Akonadi::Collection& ) ),
             im, SLOT( setCollection( const Akonadi::Collection& ) ) );

    QToolBox* toolbox = new QToolBox( this );
    toolbox->addItem( splitter, i18n("Images") );
    toolbox->addItem( m_filebrowser, i18n("Files") );

    QDockWidget* attachDock = new QDockWidget( i18n( "At&tach File" ), this );
    attachDock->setWidget( toolbox );
    attachDock->setObjectName( "attachDock" );
    attachDock->setFeatures( QDockWidget::DockWidgetMovable |
                             QDockWidget::DockWidgetClosable );
    addDockWidget( Qt::RightDockWidgetArea, attachDock );

    // Recent addresses.
    m_recentbook = new QTreeView( this );
    m_recentbook->setSortingEnabled( true );
    m_recentbook->sortByColumn( 3, Qt::DescendingOrder );
    m_recentbook->setContextMenuPolicy( Qt::CustomContextMenu );

    m_recentModel = new RecentModel( this );
    m_recentModel->fetchScope().fetchFullPayload();

    m_recentbook->setModel( m_recentModel );

    connect( m_recentbook,
             SIGNAL( clicked( const QModelIndex& ) ),
             SLOT( slotAddAddressFromRecentBook( const QModelIndex& ) ) );
    connect( m_recentbook,
             SIGNAL( customContextMenuRequested( const QPoint & ) ),
             SLOT( slotContextMenuRecentBook() ) );

    RecentManager* recentManager = RecentManager::instance();
    connect( recentManager, SIGNAL( recentCollection( Akonadi::Collection ) ),
             SLOT( slotSetRecentModel( Akonadi::Collection ) ) );
    recentManager->getRecentCollection();

    QDockWidget* recentDock = new QDockWidget( i18n( "Re&cent" ), this );
    recentDock->setWidget( m_recentbook );
    recentDock->setObjectName( "recentDock" );
    recentDock->setFeatures( QDockWidget::DockWidgetMovable |
                             QDockWidget::DockWidgetClosable );
    addDockWidget( Qt::LeftDockWidgetArea, recentDock );

    // check that we have state-settings saved. If not hide docks by default.
    KConfigGroup dcfg = KGlobal::config()->group( "Composer" );
    if ( !dcfg.hasKey( "FirstRun" ) ) {
        addressDock->setHidden( true );
        recentDock->setHidden( true );
        attachDock->setHidden( true );
        dcfg.writeEntry( "FirstRun",false );
    }

    tabifyDockWidget( recentDock, addressDock );

    // -------------- main layout ----//

    m_vsplitter = new QSplitter( this );
    m_vsplitter->setOrientation( Qt::Vertical );

    QWidget* widg = new QWidget( m_vsplitter );
    QGridLayout* grid = new QGridLayout( widg );
    grid->setSpacing( 6 );
    grid->setMargin( 6 );

    m_identitylabel = new QLabel( i18n( "Identity:" )+' ', widg );
    m_identityBox = new KPIMIdentities::IdentityCombo( Global::identityManager(),
            widg );
    m_identitylabel->setBuddy( m_identityBox );
    connect( m_identityBox,SIGNAL( identityChanged( uint ) ),
             SLOT( slotIdentityChanged( uint ) ) );

    m_dictionarylabel = new QLabel( i18n( "Dictionary:" ) + ' ', widg );
    m_dictionaryBox = new Sonnet::DictionaryComboBox( widg );
    m_dictionarylabel->setBuddy( m_dictionaryBox );
    connect( m_dictionaryBox, SIGNAL( dictionaryChanged( const QString & ) ),
             SLOT( slotDictionaryChanged( const QString& ) ) );

    m_transportlabel = new QLabel( i18n( "Transport:" )+' ', widg );
    m_transportBox = new MailTransport::TransportComboBox( widg );
    m_transportlabel->setBuddy( m_transportBox );
    connect( m_transportBox, SIGNAL( currentIndexChanged( int ) ),
             SLOT( slotTransportChanged() ) );

    m_recipientWidget = new RecipientWidget( widg, m_recentModel );
    connect( m_recipientWidget, SIGNAL( dirty() ),
             SLOT( slotSetDirty() ) );

    // signal coming from the dialog with detail info
    connect( tooltip, SIGNAL( mailClicked( const QString& ) ),
             m_recipientWidget, SLOT( slotAddAddress( const QString& ) ) );

    m_alabel = new QLabel( i18n( "Attached:" )+' ', widg );
    m_attachview = new AttachmentWidget( widg );
    connect( m_attachview, SIGNAL( noAttachmentsLeft() ),
             SLOT( slotHideAttachmentWidget() ) );

    m_attachview->hide();
    m_alabel->setBuddy( m_attachview );
    m_alabel->hide();

    grid->addWidget( m_identitylabel,0,0, Qt::AlignRight );
    grid->addWidget( m_identityBox,0,1,1,2 );
    grid->addWidget( m_dictionarylabel,1,0, Qt::AlignRight );
    grid->addWidget( m_dictionaryBox,1,1,1,2 );
    grid->addWidget( m_transportlabel,2,0, Qt::AlignRight );
    grid->addWidget( m_transportBox,2,1,1,2 );
    grid->addWidget( m_recipientWidget,3,0,1,2 );
    grid->addWidget( m_alabel,4,0, Qt::AlignRight );
    grid->addWidget( m_attachview,4,1,1,2 );
    grid->setColumnStretch( 1,10 );

    //--------------------------------------------------------

    QWidget* widg2 = new QWidget( m_vsplitter );
    QGridLayout* grid2 = new QGridLayout( widg2 );
    grid2->setSpacing( 5 );
    grid2->setMargin( 5 );

    QLabel* label = new QLabel( i18n( "Subject:" )+' ', widg );
    m_subject = new KLineEdit( widg );
    m_subject->setObjectName( "subject" );
    label->setBuddy( m_subject );
    connect( m_subject, SIGNAL( textChanged( const QString & ) ),
             SLOT( slotSetDirty() ) );

    m_text = new ComposerTextEdit( widg2 );
    m_text->setRichTextSupport( ComposerTextEdit::FullSupport );
    m_text->createActions( actionCollection() );
    connect( m_text, SIGNAL( textChanged() ), SLOT( slotSetDirty() ) );
    connect( m_text, SIGNAL( addAttachment( const KUrl& ) ),
             SLOT( slotAddFile( const KUrl& ) ) );
    connect( m_text, SIGNAL( languageChanged( const QString & ) ),
             SLOT( slotDictionaryConfigChanged( const QString& ) ) );

    KStandardAction::preferences( this, SLOT( slotSetup() ), actionCollection() );

    KAction* action = new KAction( i18n( "Check &spelling..." ), this );
    action->setIconText( i18n( "Check Spelling" ) );
    actionCollection()->addAction( "options_configure_spell", action );
    connect( action, SIGNAL( triggered( bool ) ), SLOT( slotSpell() ) );

    actionCollection()->addAction( KStandardAction::Spelling, "spellcheck",
                                   m_text, SLOT( checkSpelling() ) );

    m_textToolBar = new KToolBar( "htmlToolBar", this, Qt::TopToolBarArea );
    m_textToolBar->setToolButtonStyle( Qt::ToolButtonIconOnly );

    m_htmlmode = new KToggleAction( this );
    m_htmlmode->setText( i18n( "HTML Mode" ) );
    actionCollection()->addAction( "toggle_html_mode", m_htmlmode );
    connect( m_htmlmode, SIGNAL( toggled( bool ) ), SLOT( slotHTMLMode( bool ) ) );

    uint identityInt = m_identityBox->currentIdentity();
    const KPIMIdentities::Identity& ident =
        Global::identityManager()->identityForUoidOrDefault( identityInt );
    bool htmlActive = ident.property( s_preferhtml ).toInt() == 1 ? false : true;
    m_htmlmode->setChecked( htmlActive );
    slotHTMLMode( htmlActive );

    m_dictionaryBox->setCurrentByDictionaryName( ident.dictionary() );
    m_text->setSpellCheckingLanguage( m_dictionaryBox->currentDictionary() );

    m_sendmail = new KAction( this );
    m_sendmail->setText( i18n( "Send Email" ) );
    m_sendmail->setIcon( KIcon( "mail-send" ) );
    m_sendmail->setShortcut( Qt::ControlModifier + Qt::Key_Return );
    m_sendmail->setIcon( KIcon( "mail-send" ) );
    actionCollection()->addAction( "send_message", m_sendmail );
    connect( m_sendmail, SIGNAL( triggered( bool ) ), SLOT( slotSend() ) );

    m_savemail = new KAction( this );
    m_savemail->setText( i18n( "Save Email" ) );
    m_savemail->setIcon( KIcon( "document-save" ) );
    m_savemail->setShortcut( Qt::ControlModifier + Qt::Key_S );
    actionCollection()->addAction( "composer_save_message", m_savemail );
    connect( m_savemail, SIGNAL( triggered( bool ) ), SLOT( slotSave() ) );

    m_expandSig = new KAction( this );
    m_expandSig->setText( i18n( "Expand Signature" ) );
    actionCollection()->addAction( "expand_sig", m_expandSig );
    connect( m_expandSig, SIGNAL( triggered( bool ) ), SLOT( slotExpandSignature() ) );

    m_showIdentities = new KToggleAction( this );
    m_showIdentities->setText( i18n( "Show Identities" ) );
    actionCollection()->addAction( "options_show_identities", m_showIdentities );
    connect( m_showIdentities, SIGNAL( triggered( bool ) ),
             SLOT( slotShowIdentities( bool ) ) );

    m_showDictionaries = new KToggleAction( this );
    m_showDictionaries->setText( i18n( "Show Dictionaries" ) );
    actionCollection()->addAction( "options_show_dictionaries", m_showDictionaries );
    connect( m_showDictionaries, SIGNAL( triggered( bool ) ),
             SLOT( slotShowDictionaries( bool ) ) );

    m_showTransports = new KToggleAction( this );
    m_showTransports->setText( i18n( "Show Transports" ) );
    actionCollection()->addAction( "options_show_transports", m_showTransports );
    connect( m_showTransports, SIGNAL( triggered( bool ) ),
             SLOT( slotShowTransports( bool ) ) );

    KAction *close = new KAction( this );
    close->setText( i18n( "Close" ) );
    close->setIcon( KIcon( "window-close" ) );
    close->setShortcut( KStandardShortcut::close() );
    actionCollection()->addAction( "composer_close", close );
    connect( close, SIGNAL( triggered( bool ) ), SLOT( slotDone() ) );

    KToggleAction *toggleFixed = new KToggleAction( this );
    toggleFixed->setText( i18n( "Fixed Font" ) );
    actionCollection()->addAction( "toggle_fixed_font", toggleFixed );
    connect( toggleFixed, SIGNAL( triggered( bool ) ), SLOT( slotFixedFont( bool ) ) );

    actionCollection()->addAction( "toggle_address_dock",
                                   addressDock->toggleViewAction() );
    actionCollection()->addAction( "toggle_recent_dock",
                                   recentDock->toggleViewAction() );
    actionCollection()->addAction( "toggle_attach_dock",
                                   attachDock->toggleViewAction() );


    QToolButton *send = new QToolButton( widg2 );
    send->setDefaultAction( m_sendmail );

    QToolButton *save = new QToolButton( widg2 );
    save->setDefaultAction( m_savemail );

    grid2->addWidget( label,0,0, Qt::AlignRight );
    grid2->addWidget( m_subject,0,1,1,4 );
    grid2->addWidget( m_textToolBar,1,0,1,5 );
    grid2->addWidget( m_text,2,0,1,5 );
    grid2->addWidget( send,3,4 );
    grid2->addWidget( save,3,3 );
    grid2->setColumnStretch( 2,10 );

    //--------------------------------------------------------

    m_vsplitter->show();
    setCentralWidget( m_vsplitter );
    setMinimumSize( QSize( 400,300 ) );
    setStandardToolBarMenuEnabled( true );
    toolBar();
    statusBar();
    setupGUI( Keys | StatusBar | ToolBar | Create | Save, "composerui.rc" );

    // set the column width of the subject to the same value
    // as the labels above the splitter. Not sure why 3*spacing is needed.
    m_identitylabel->adjustSize();
    int width = m_identitylabel->width() + ( 3*grid2->spacing() );
    grid2->setColumnMinimumWidth( 0, width );

    // read config for the splitter size
    setAutoSaveSettings( "Composer", true );
    KConfigGroup config = KGlobal::config()->group( "ComposerSplitterSize" );
    slotShowIdentities( config.readEntry( "showIdentities", false ) );
    slotShowDictionaries( config.readEntry( "showDictionaries", false ) );
    slotShowTransports( config.readEntry( "showTransports", false ) );

    QList<int> defaultsize = config.readEntry( "vsplitter",QList<int>() );
    if ( !defaultsize.count() ) {
        defaultsize.append( 200 );
        defaultsize.append( 400 );
    }
    m_vsplitter->setSizes( defaultsize );

    // When we have time, load the addressess from ab
    QTimer::singleShot( 0, this, SLOT( slotLoadAddressBook() ) );

    // Statusbar stuff
    statusBar()->insertItem( QString(), 1, 10 );
    statusBar()->insertItem( m_identityBox->currentIdentityName(), 2, 1 );
    statusBar()->insertItem( QString(), 3, 1 );
    slotTransportChanged();

    connect( statusBar(), SIGNAL( pressed( int ) ),
             SLOT( slotStatusBarClicked( int ) ) );
}

Composer::~Composer()
{
    KConfigGroup config = KGlobal::config()->group( "ComposerSplitterSize" );
    config.writeEntry( "vsplitter", m_vsplitter->sizes() );
    config.writeEntry( "showIdentities", m_showIdentities->isChecked() );
    config.writeEntry( "showDictionaries", m_showDictionaries->isChecked() );
    config.writeEntry( "showTransports", m_showTransports->isChecked() );
    config.sync();
}

void Composer::slotSetRecentModel( Collection collection )
{
    m_recentModel->setCollection( collection );
}

// composer
void Composer::slotStatusBarClicked( int item )
{
    QHash<QAction*, QString> temp;
    QMenu *p = new QMenu( this );

    // item 2 is identity and 3 is the transport.
    if ( item ==2 ) {
        foreach( const QString& identity, Global::identityManager()->identities() ) {
            QAction* tmp = new QAction( identity, this );
            p->addAction( tmp );
            temp[tmp] = identity;
        }
    } else if ( item ==3 ) {
        foreach( const QString& transport, MailTransport::TransportManager::self()->transportNames() ) {
            QAction *tmp = new QAction( transport, this );
            p->addAction( tmp );
            temp[tmp] = transport;
        }
    } else {
        return;
    }

    QAction* choice = p->exec( QCursor::pos() );
    const QString newItem = temp.value( choice );

    if ( item == 2 ) {
        m_identityBox->setCurrentIdentity( newItem );
    } else if ( item ==3 ) {
        int id = MailTransport::TransportManager::self()->transportByName( newItem )->id();
        m_transportBox->setCurrentTransport( id );
    }
}
void Composer::slotHideAttachmentWidget()
{
    m_attachview->hide();
    m_alabel->hide();
}

void Composer::slotShowIdentities( bool on )
{
    m_identitylabel->setVisible( on );
    m_identityBox->setVisible( on );
    m_showIdentities->setChecked( on ); // when reading from config...
}

void Composer::slotShowDictionaries( bool on )
{
    m_dictionarylabel->setVisible( on );
    m_dictionaryBox->setVisible( on );
    m_showDictionaries->setChecked( on ); // when reading from config...
}

void Composer::slotShowTransports( bool on )
{
    m_transportlabel->setVisible( on );
    m_transportBox->setVisible( on );
    m_showTransports->setChecked( on ); // when reading from config...
}

void Composer::setDirty( bool dirty )
{
    m_dirty = dirty;
}

bool Composer::queryClose()
{
    if ( m_close )
        return true;

    return canClose();
}

bool Composer::canClose()
{
    if ( m_dirty ) {
        int res = KMessageBox::questionYesNo( this,
                                              i18n( "You have unsaved changes, do you want to save the draft?" ) );
        if ( res == KMessageBox::Yes ) {
            slotSave();
            return false; /* slotSave will close it if needed */
        } else
            return true;
    }
    return true;
}

void Composer::setIdentity( KPIMIdentities::Identity identity )
{
    m_identityBox->setCurrentIdentity( identity );
}

void Composer::setSubject( const QString& subject )
{
    m_subject->blockSignals( true );
    m_subject->setText( subject );
    m_origSubject = subject;
    m_subject->blockSignals( false );
}

void Composer::setMsgAndQuote( const QString& plain, const QString& html, bool sign )
{
    if ( m_text->textMode()==KRichTextWidget::Plain ) {
        QString plaintext = i18n( "You wrote:\n" ) + Global::quote( plain ) + '\n';
    }
    setMsg( plain, html, sign );

    bool dirty = m_dirty;
    // Add header.
    m_text->insertHtml( i18n( "<br><hr><b>You wrote:</b><br>" ) );

    // Move to top
    QTextCursor cursor = m_text->textCursor();
    cursor.setPosition( 0 );
    m_text->setTextCursor( cursor );
    setDirty( dirty );
}

void Composer::setMsg( const QString& plain, const QString& html, bool sign )
{
    QString message = ( m_text->textMode()==KRichTextWidget::Rich ) ? html : plain;

    if ( sign )
        addSignaturePlaceholder( message );

    // we do not want to get this dirty
    bool dirty = m_dirty;
    m_text->setTextOrHtml( message );
    // TODO:    ( m_addressbox->topLevelItemCount() > 0 ) ? m_text->setFocus() : m_edit->setFocus();
    setDirty( dirty );
}

void Composer::addAttachment( const KUrl& attachment, const QString& name )
{
    m_attachview->show();
    m_alabel->show();
    m_attachview->addAttachment( attachment, name );
}

void Composer::inReplyTo( const QString& messageID )
{
    m_replyTo = messageID;
}


void Composer::addSignaturePlaceholder( QString& text )
{
    uint signWith = m_identityBox->currentIdentity();
    KPIMIdentities::Identity ident =
        Global::identityManager()->identityForUoidOrDefault( signWith );
    if ( ident.signature().type() == KPIMIdentities::Signature::Disabled )
        return;

    int pos = ident.property( s_sigpos ).toInt();
    pos == 2 ?  text.prepend( "@@sig@@\n\n" ) : text.append( "@@sig@@" );
}

void Composer::placeSignature( QString& text )
{
    uint signWith = m_identityBox->currentIdentity();
    KPIMIdentities::Identity ident =
        Global::identityManager()->identityForUoidOrDefault( signWith );

    KPIMIdentities::Signature sig = ident.signature();

    // Fetch the signature.
    QString signature = sig.withSeparator();

    // It will be empty if disabled.
    if ( signature.isEmpty() ) {
        text = text.remove( "@@sig@@" );
        return;
    }

    // Replace the placeholder with the signature.
    text = text.replace( "@@sig@@", signature );
}

void Composer::slotIdentityChanged( uint uid )
{
    // change status bar first...
    statusBar()->changeItem( m_identityBox->currentIdentityName(), 2 );

    // change spellcheck
    const KPIMIdentities::Identity& ident =
        Global::identityManager()->identityForUoidOrDefault( uid );
    m_dictionaryBox->setCurrentByDictionaryName( ident.dictionary() );
    m_text->setSpellCheckingLanguage( m_dictionaryBox->currentDictionary() );

    // if the signature is expanded already, we don't want this.
    if ( !m_expandSig->isEnabled() )
        return;

    // Remove the current placeholder
    // TODO: The toPlainText() here and setText() below destroy all formatting
    QString msg = m_text->toPlainText();
    msg.remove( "@@sig@@" );

    // Add a new one if needed.
    addSignaturePlaceholder( msg );

    // Set the text in the box again, but don't trigger dirty.
    bool dirty = m_dirty;
    m_text->setText( msg );
    setDirty( dirty );
}

void Composer::slotAddFile( const QModelIndex & index )
{
    KDirModel* model = static_cast<KDirModel*>( m_filebrowser->model() );
    KFileItem file = model->itemForIndex( index );
    if ( !file.isFile() )
        return;

    // find existing one
    QList<KUrl> list = m_attachview->attachments().keys();
    foreach( const KUrl& url, list ) {
        if ( url.path() == file.url().path() )
            return;
    }

    // Add it...
    setDirty( true ); // do it here, as forward message does not come here
    addAttachment( file.url() );
}

void Composer::slotImageClicked( const QModelIndex& index )
{
    const ImageModel* model = static_cast<const ImageModel*>( index.model() );
    QString filename = model->itemForIndex( index ).remoteId();
    slotAddFile( filename );
}

void Composer::slotAddFile( const KUrl& file )
{
    setDirty( true );
    addAttachment( file );
}

void Composer::setRcpt( const QString& address,
                        RecipientWidget::TypeOfAddress addressType )
{
    m_recipientWidget->setRcpt( address, addressType );
}

// -------------- addressbook ---------------------------

void Composer::slotAddAddressFromAddressBook( const QModelIndex& index )
{
    Akonadi::Item item = index.data( ContactTreeModel::ItemRole ).value<Akonadi::Item>();
    if ( item.mimeType() == "text/directory" ) {
        m_recipientWidget->slotAddAddress( index.data( ContactTreeModel::AddressRole ).toString() );
    } else if ( item.mimeType() == "application/x-vnd.kde.contactgroup" ) {
        const KABC::ContactGroup group = item.payload<KABC::ContactGroup>();
        Akonadi::ContactGroupExpandJob *job = new Akonadi::ContactGroupExpandJob( group );
        connect( job, SIGNAL( result( KJob* ) ), this, SLOT( expandResult( KJob* ) ) );
        job->start();
    }
}

void Composer::expandResult( KJob *job )
{
    Akonadi::ContactGroupExpandJob *expandJob = qobject_cast<Akonadi::ContactGroupExpandJob*>( job );
    const KABC::Addressee::List contacts = expandJob->contacts();
    foreach( const KABC::Addressee &contact, contacts ) {
        m_recipientWidget->slotAddAddress( contact.fullEmail() );
    }
}


void Composer::slotContextMenuAddressBook()
{
    const QModelIndex lvi = m_addressbookView->currentIndex();
    if ( !lvi.isValid() )
        return;

    QMenu* p = new QMenu( this );
    QAction* edit = new QAction( i18n( "Edit contact" ), this );
    QAction* del = new QAction( i18n( "Delete from address book" ), this );

    p->addAction( edit );
    p->addAction( del );
    QAction* choice = p->exec( QCursor::pos() );

    if ( choice == del ) {
        const Akonadi::Item item = lvi.data( Akonadi::EntityTreeModel::ItemRole ).value<Akonadi::Item>();
        if ( !item.isValid() )
            return;

        Akonadi::ItemDeleteJob *job = new Akonadi::ItemDeleteJob( item );
        if ( !job->exec() )
            kDebug() << "Error occurred" << job->errorString();
    } else if ( choice == edit ) {
        const Akonadi::Item item = lvi.data( Akonadi::EntityTreeModel::ItemRole ).value<Akonadi::Item>();
        if ( !item.isValid() )
            return;

        Akonadi::ContactEditorDialog dlg( Akonadi::ContactEditorDialog::EditMode, this );
        dlg.setContact( item );

        if ( dlg.exec() ) {
            qDebug() << "Contact has been edited";
        } else {
            qDebug() << "User has canceled operation";
        }
    }
}

void Composer::slotContextMenuAddressList()
{
    /* TODO
    using namespace KABC;

    QTreeWidgetItem* lvi = m_addressbox->currentItem();
    if ( !lvi )
        return;

    QMenu* p = new QMenu( this );
    QAction* add = new QAction( i18n( "Add to KAddressBook" ), this );

    p->addAction( add );
    QAction* choice = p->exec( QCursor::pos() );

    if ( choice == add )
        Addressbook::instance()->add( lvi->text( 0 ) );
        */
}

// ---------------------- recent -----------------------//

void Composer::slotAddAddressFromRecentBook( const QModelIndex& lvi )
{
    if ( !lvi.isValid() )
        return;

    QModelIndex email = m_recentbook->model()->index( lvi.row(), 0, lvi.parent() );
    QModelIndex name = m_recentbook->model()->index( lvi.row(), 1, lvi.parent() );
    m_recipientWidget->slotAddAddress( m_recentbook->model()->data( name ).toString() + " <" +
                                       m_recentbook->model()->data( email ).toString() + '>' );
}

void Composer::slotContextMenuRecentBook()
{
    const QModelIndex lvi = m_recentbook->currentIndex();
    if ( !lvi.isValid() )
        return;

    QMenu* p = new QMenu( this );
    QAction* del = new QAction( i18n( "Delete" ), this );
    p->addAction( del );
    QAction* choice = p->exec( QCursor::pos() );

    if ( choice == del ) {
        RecentModel* model = static_cast<RecentModel*>( m_recentbook->model() );
        const Akonadi::Item item = model->itemForIndex( lvi );
        if ( !item.isValid() )
            return;

        Akonadi::ItemDeleteJob *job = new Akonadi::ItemDeleteJob( item );
        if ( !job->exec() )
            kDebug() << "Error occurred" << job->errorString();
    }
}

// ---------------------- send -------------------------//

void Composer::addRecipients( Mailody::SendMessage* msg ) //  There's a SendMessage in Win32 API, using Mailody::SendMessage to disambiguate
{
    msg->setTos( m_recipientWidget->list( RecipientWidget::To ) );
    msg->setCcs( m_recipientWidget->list( RecipientWidget::Cc ) );
    msg->setBccs( m_recipientWidget->list( RecipientWidget::Bcc ) );
}

void Composer::slotSend()
{
    int current = m_transportBox->currentTransportId();
    if ( current == -1 ) {



        // TODDO: PORT TO createTransport call with enum for if not exists. Probably in the constructor of the composer.



        KMessageBox::information( this, i18n( "There is no outgoing mailserver configuration found, please set it up before sending" ) );
        slotSetup();
        return;
    }

    if ( !m_recipientWidget->hasRecipients() ) {
        KMessageBox::information( this, i18n( "No recipients added. Please add at least one before sending." ) );
        m_recipientWidget->setFocus();
        return;
    }

    if ( !m_origSubject.isEmpty() && m_origSubject != m_subject->text() &&
            !m_replyTo.isEmpty() ) {
        int i = KMessageBox::questionYesNo( 0,
                                            i18n( "You have changed the subject in the thread you "
                                                  "are writing to, do you want to start a new thread?" ),
                                            i18n( "Subject Change" ),
                                            KStandardGuiItem::yes(), KStandardGuiItem::no(),
                                            "break_threads_on_subject_change" );
        if ( i == KMessageBox::Yes )
            m_replyTo.clear();
    }

    // expand signature, since we are sending now.
    slotExpandSignature();

    SendMessage* toSend = new SendMessage( this );
    connect( toSend, SIGNAL( finished() ), SLOT( slotDone() ) );
    connect( toSend, SIGNAL( sendError( const QString& ) ),
             SLOT( slotError( const QString& ) ) );

    addRecipients( toSend );
    toSend->addAttachments( m_attachview->attachments() );

    QString plain= m_text->toCleanPlainText();
    QString html;
    if ( m_text->isFormattingUsed() )
        html = m_text->toCleanHtml();

    toSend->setSubject( m_subject->text() );
    toSend->send( m_mailbox, m_identityBox->currentIdentity(),
                  m_transportBox->currentTransportId(), plain, html );
}

void Composer::slotSave()
{


    // PORT to use the special folders API


    if ( !m_mailbox.isValid() ) {
        kWarning() << m_mailbox.name() << " is not valid. Not saving.";
        return;
    }

    SendMessage* toSend = new SendMessage( this );
    addRecipients( toSend );
    toSend->addAttachments( m_attachview->attachments() );

    QString plain= m_text->toCleanPlainText();
    QString html;
    if ( m_text->isFormattingUsed() )
        html = m_text->toCleanHtml();

    KMime::Message* kmimeMessage = toSend->compileMessage( SendMessage::Draft,
                                   m_identityBox->currentIdentity(), plain, html );
    const QString flags = "\\Draft";

    if ( !kmimeMessage->hasContent() ) {
        kWarning() << "Message has no content, aborting";
        return;
    }

    Akonadi::Item item( "message/rfc822" );
    item.setPayload( KMime::Message::Ptr( kmimeMessage ) );

    Akonadi::ItemCreateJob* job = new Akonadi::ItemCreateJob( item, m_mailbox, 0 );
    connect( job, SIGNAL( result( KJob* ) ), this, SLOT( slotDone( KJob* ) ) );
}

void Composer::slotFixedFont( bool toggle )
{
    toggle ? m_text->setFont( KGlobalSettings::fixedFont() )
    : m_text->setFont( KGlobalSettings::generalFont() );
}

void Composer::slotExpandSignature()
{
    m_expandSig->setEnabled( false );

    QString msg = m_text->textOrHtml();
    placeSignature( msg );

    // if this is rich text, switch to rich text mode first...
    ComposerTextEdit::Mode mode = m_text->textMode();
    bool dirty = m_dirty;
    m_text->setTextOrHtml( msg );
    setDirty( dirty );

    // If the signature is made up from html, the textedit is now
    // automatically set to html mode, revert that.
    // It kind of sucks, because formatting could be lost here.
    if ( mode == ComposerTextEdit::Plain && m_text->textMode() == ComposerTextEdit::Rich ) {
        m_text->switchToPlainText();
    }
}

void Composer::slotHTMLMode( bool toggle )
{
    if ( !toggle && !m_text->toPlainText().isEmpty() && m_text->isFormattingUsed() ) {
        int choice = KMessageBox::warningContinueCancel( this, i18n( "Switching from HTML mode "
                     "to Plain mode will cause the text to lose the formatting. Are you sure?" ),
                     i18n( "Lose the formatting?" ), KStandardGuiItem::cont(), KStandardGuiItem::cancel(),
                     "switchhtml2plainisok" );
        if ( choice != KMessageBox::Continue ) {
            m_htmlmode->setChecked( true );
            return;
        }
    }
    m_textToolBar->setEnabled( toggle );
    m_textToolBar->setVisible( toggle );

    toggle ? m_text->enableRichTextMode() : m_text->switchToPlainText();
}

void Composer::slotSetup()
{
    emit showSettings();
}

void Composer::slotSetDirty()
{
    setDirty( true );
}

void Composer::slotDone()
{
    setDirty( false );
    close();
}

void Composer::slotDone( KJob* job )
{
    //TODO: KMessageBox...
    QApplication::restoreOverrideCursor();
    if ( job->error() ) {
        kWarning() << job->errorString();
    } else {
        m_close = true;
        close();
    }
}

void Composer::slotError( const QString& error )
{
    QApplication::restoreOverrideCursor();
    KMessageBox::information( this, error );
}

void Composer::slotSpell()
{
    m_text->showSpellConfigDialog( "mailodyrc" );
}

void Composer::slotDictionaryConfigChanged( const QString& dict )
{
    // This happens when the sonnet config changes.
    // the editor already knows the new language, so block the signals
    // else the slotDictionaryChanged() would be called and sets the lang a second time...

    m_dictionaryBox->blockSignals( true );
    m_dictionaryBox->setCurrentByDictionary( dict );
    m_dictionaryBox->blockSignals( false );

    // save dictionary in the identity.
    uint identityInt = m_identityBox->currentIdentity();
    KPIMIdentities::Identity& ident = Global::identityManager()->modifyIdentityForUoid( identityInt );
    ident.setDictionary( m_dictionaryBox->currentDictionaryName() );
    Global::identityManager()->commit();
}

void Composer::slotDictionaryChanged( const QString& )
{
    m_text->setSpellCheckingLanguage( m_dictionaryBox->currentDictionary() );

    // save dictionary in the identity.
    uint identityInt = m_identityBox->currentIdentity();
    KPIMIdentities::Identity& ident = Global::identityManager()->modifyIdentityForUoid( identityInt );
    ident.setDictionary( m_dictionaryBox->currentDictionaryName() );
    Global::identityManager()->commit();
}

void Composer::slotTransportChanged()
{
    int current = m_transportBox->currentTransportId();
    if ( current == -1 ) {
        statusBar()->changeItem( i18nc( "No valid mail transport selected","None" ), 3 );
        return;
    }

    // set the status bar.
    const QString t = MailTransport::TransportManager::self()->transportById( current )->name();
    statusBar()->changeItem( t, 3 );
}

#include "composer.moc"
