/* This file is part of the KDE project

   Copyright (C) 2006-2007 Omat Holding B.V. <info@omat.nl>
   Copyright (C) 2007 Frode M. Døving <frode@lnix.net>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*/


#ifndef COMPOSER_H
#define COMPOSER_H

#include <akonadi/collection.h>
#include "composertextedit.h"
#include <kxmlguiwindow.h>
#include "mailtransport/transportcombobox.h"
#include "recipientwidget.h" // Enum

class QLabel;
class QSplitter;
class QTreeView;

class KAction;
class KJob;
class KLineEdit;
class KToggleAction;
class KUrl;

namespace KMime
{
class Message;
}

namespace KPIMIdentities
{
class Identity;
class IdentityCombo;
}

namespace Sonnet
{
class DictionaryComboBox;
}

namespace Akonadi
{
class ItemModel;
}

namespace Mailody
{
class SendMessage;
class AttachmentWidget;


/**
 * @class Composer
 * This is het main class for the composer...
 * @author Tom Albers <tomalbers@kde.nl>
 */
class Composer : public KXmlGuiWindow
{
    Q_OBJECT

public:
    /**
    * Contructor
    * @param parent the parent widget.
    * @param mb the mailbox where the resulting message will be stored.
    */
    explicit Composer( QWidget* parent, const Akonadi::Collection& mb );

    /**
    * Destructor
    */
    ~Composer();

    /**
     * Call this to find out if the window is dirty
     */
    bool canClose();

    /**
     * Use a certain identity
     * @param identity the identity to use.
     */
    void setIdentity( KPIMIdentities::Identity identity );

    /**
     * Use a certain subject
     * @param subject the subject to use.
     */
    void setSubject( const QString& subject );

    /**
     * set @p plain or @p html to be the message, after this call the cursor
     * will be set to the top of the text part, but only when there is
     * at least one recipient. @p sign indicates if you want to sign the
     * or not. Defaults to true (sign).
     *
     * Whether @p plain or @p html is used, depends on the seting of the composer.
     * @param plain The plain text.
     * @param html The rich text.
     * @param sign Set to true to add signature markers.
     */
    void setMsg( const QString& plain, const QString& html = QString(), bool sign=true );

    /**
      * similar to above, but it will quote the message, html and plain
     * @param plain The plain text.
     * @param html The rich text.
     * @param sign Set to true to add signature markers.
      */
    void setMsgAndQuote( const QString& plain, const QString& html, bool sign=true );

    /**
     * add attachment to the message
     * @param attachment The url to the attachment.
     * @param name The user visible name.
     */
    void addAttachment( const KUrl& attachment, const QString& name=QString() );

    /**
     * set a In-reply-to header
     * @param messageID The id
     */
    void inReplyTo( const QString& messageID );

    /**
     * add signature placeholder
     */
    void addSignaturePlaceholder( QString& );

    /**
     * adds a recipient
     */
    void setRcpt( const QString& address, RecipientWidget::TypeOfAddress addressType );

    /**
     * set composer dirty or not.
     * @param dirty Set to true to make it dirty.
     */
    void setDirty( bool dirty=true );

protected:
    /** Reimplemented to catch if there are any unsaved modifications. */
    bool queryClose();

// @cond private
private:
    QTreeView                       *m_addressbookView;
    KAction                         *m_expandSig;
    KAction                         *m_sendmail;
    KAction                         *m_savemail;
    KToggleAction                   *m_htmlmode;
    KToolBar                        *m_textToolBar;
    KToggleAction                   *m_showIdentities;
    KToggleAction                   *m_showDictionaries;
    KToggleAction                   *m_showTransports;
    QTreeView                       *m_recentbook;
    RecipientWidget*                m_recipientWidget;
    QMap<QTreeWidgetItem*, QString> m_abMap;
    QSplitter*                      m_vsplitter;
    KLineEdit*                      m_subject;
    QString                         m_origSubject;
    ComposerTextEdit*               m_text;
    KMime::Message*                 m_main;
    QStringList                     m_tos;
    Akonadi::Collection             m_mailbox;
    QString                         m_replyTo;
    bool                            m_close;
    bool                            m_dirty;
    KPIMIdentities::IdentityCombo   *m_identityBox;
    QLabel                          *m_identitylabel;
    QLabel                          *m_dictionarylabel;
    Sonnet::DictionaryComboBox      *m_dictionaryBox;
    MailTransport::TransportComboBox *m_transportBox;
    QLabel                          *m_transportlabel;
    QTreeView*                      m_filebrowser;
    AttachmentWidget*               m_attachview;
    QLabel*                         m_alabel;
    QTreeWidgetItem*                m_currentItem;
    Akonadi::ItemModel*             m_recentModel;

    const char * encoding( const QString& data );
    void showError( const QString& error );
    void addRecipients( SendMessage* msg );
    void placeSignature( QString& );

private slots:
    void slotSetRecentModel( Akonadi::Collection collection );

    void slotStatusBarClicked( int item );
    void slotAddFile( const QModelIndex & index );
    void slotImageClicked( const QModelIndex& index );
    void slotAddFile( const KUrl& );
    void slotHTMLMode( bool );

    void slotIdentityChanged( uint signWith );
    void slotExpandSignature();

    void slotAddAddressFromAddressBook( const QModelIndex& );
    void slotContextMenuAddressBook();
    void slotContextMenuAddressList();
    void expandResult( KJob *job );

    void slotAddAddressFromRecentBook( const QModelIndex& );
    void slotContextMenuRecentBook();

    void slotSetup();
    void slotSend();
    void slotSave();
    void slotDone();
    void slotDone( KJob* );
    void slotError( const QString& error );
    void slotSetDirty();
    void slotFixedFont( bool );
    void slotTransportChanged();
    void slotDictionaryConfigChanged( const QString& );
    void slotDictionaryChanged( const QString& );
    void slotHideAttachmentWidget();
    void slotShowIdentities( bool );
    void slotSpell();
    void slotShowDictionaries( bool );
    void slotShowTransports( bool );
// @endcond

signals:
    /**
    * Will be emitted when configure mailody is triggered from the composer.
    */
    void showSettings();

};

}

#endif // COMPOSER
