/*
   Copyright (C) 2007-2009 Omat Holding B.V. <info@omat.nl>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*/

// Own
#include "headerview.h"
#include "headerproxy.h"
#include "headerdelegate.h"

// Qt
#include <QApplication>
#include <QPixmap>
#include <QPainter>

// KDE
#include <KDebug>

#include <akonadi/kmime/messagemodel.h>
#include <akonadi/control.h>
#include <akonadi/entitytreemodel.h>

using namespace Mailody;

HeaderView::HeaderView( QWidget * parent )
        : EntityTreeView( parent )
{
    setAlternatingRowColors( true );
    setContentsMargins( 0, 0, 0, 0 );
    setDragDropMode( QAbstractItemView::DragOnly );
    setDragEnabled( true );
    setAcceptDrops( false );
    setSortingEnabled( true );
    setSelectionMode( QAbstractItemView::ExtendedSelection );

    HeaderDelegate* delegate = new HeaderDelegate( this );
    setItemDelegate( delegate );

    Akonadi::Control::widgetNeedsAkonadi( this );
}

void HeaderView::mousePressEvent( QMouseEvent * event )
{
    event->accept();
    Akonadi::EntityTreeView::mousePressEvent( event );

    // bail out when the user clicks on an empty area of the view.
    const QModelIndex index = indexAt( event->pos() );
    if ( !index.isValid() )
        return;

    const Akonadi::Item item = index.data( Akonadi::EntityTreeModel::ItemRole ).value<Akonadi::Item>();

    if ( !item.isValid() )
        return;

    if ( event->button() == Qt::MidButton )
        emit selectMessageMiddle( item );
    else if ( event->button() == Qt::RightButton )
        emit selectMessageRight( item );
}

void HeaderView::keyPressEvent( QKeyEvent* e )
{
    if ( e->key() == Qt::Key_Left ) {
        e->accept();
        const QModelIndex above = indexAbove( currentIndex() );
        if ( !above.isValid() )
            return;
        selectionModel()->setCurrentIndex( above, QItemSelectionModel::SelectCurrent | QItemSelectionModel::Rows );
        scrollTo( above,QAbstractItemView::PositionAtCenter );
        emit goActive( above );
    } else if ( e->key() == Qt::Key_Right ) {
        e->accept();
        next();
    } else if ( e->key() == Qt::Key_Down ) {
        e->accept();
        emit actionRequested( MessageBodyWidget::ScrollDown );
    } else if ( e->key() == Qt::Key_Up ) {
        e->accept();
        emit actionRequested( MessageBodyWidget::ScrollUp );
    } else if ( e->key() == Qt::Key_PageDown ) {
        e->accept();
        emit actionRequested( MessageBodyWidget::ScrollPageDown );
    } else if ( e->key() == Qt::Key_PageUp ) {
        e->accept();
        emit actionRequested( MessageBodyWidget::ScrollPageUp );
    } else
        Akonadi::EntityTreeView::keyPressEvent( e );
}

void HeaderView::next()
{
    const QModelIndex below = indexBelow( currentIndex() );
    if ( !below.isValid() )
        return;
    selectionModel()->clearSelection();
    selectionModel()->setCurrentIndex( below, QItemSelectionModel::SelectCurrent | QItemSelectionModel::Rows );
    scrollTo( below,QAbstractItemView::PositionAtCenter );
    emit goActive( below );
}

bool HeaderView::nextUnread()
{
    const QSortFilterProxyModel* proxy =
        static_cast<QSortFilterProxyModel*>( model() );
    const Akonadi::MessageModel* model = static_cast<Akonadi::MessageModel*>( proxy->sourceModel() );

    QModelIndex start = currentIndex();
    QModelIndex below = indexBelow( currentIndex() );

    // if we have no current index...
    if ( !start.isValid() ) {
        start = below = proxy->index( 0, 0 );
    }

    while ( below.isValid() ) {
        const Akonadi::Item item = model->itemForIndex( proxy->mapToSource( below ) );
        if ( !item.hasFlag( "\\Seen" ) ) {
            selectionModel()->setCurrentIndex( below, QItemSelectionModel::SelectCurrent | QItemSelectionModel::Rows );
            scrollTo( below,QAbstractItemView::PositionAtCenter );
            goActive( below );
            return true;
        }

        below = indexBelow( below );

        // see if we reached the bottom...
        if ( !below.isValid() )
            below = proxy->index( 0, 0 );

        // if we are at our starting point, this is it, no luck
        if ( below.row() == start.row() )
            break;
    }
    return false;
}

QList<int> HeaderView::columnSizes()
{
    QList<int> sizes;
    if ( !model() )
        return sizes; /* before the first folder...*/

    for ( int i = 0; i < model()->columnCount(); ++i )
        sizes.append( columnWidth( i ) );
    return sizes;
}

void HeaderView::setColumnSizes( QList<int> sizes )
{
    for ( int i = 0; i < sizes.size(); ++i )
        setColumnWidth( i, sizes.at( i ) );
}

void HeaderView::openItem( const QModelIndex & index )
{
    static const HeaderProxy *headerProxy =
        static_cast<const HeaderProxy*>( model() );
    setExpanded( headerProxy->mapFromSource( index ), true );
}

#include "headerview.moc"
