/*
   Copyright (C) 2006-2009 Omat Holding B.V. <info@omat.nl>
   Copyright (C) 2006-2007 Frode M. Døving <frode@lnix.net>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*/

// Own
#include "mainwindow.h"

#include "collectiondelegate.h"
#include "composer.h"
#include "flagmodifyjob.h"
#include "global.h"
#include "headerview.h"
#include "messagewidget.h"
#include "messageheaderwidget.h"
#include "quickreplywidget.h"
#include "setup.h"
#include "systemtray.h"
#include "tabwidget.h"

// akonadi
#include <akonadi/changerecorder.h>
#include <akonadi/collection.h>
#include <akonadi/collectionmodel.h>
#include <akonadi/control.h>
#include <akonadi/entitytreemodel.h>
#include <akonadi/entitytreeview.h>
#include <akonadi/entitytreeviewstatesaver.h>
#include <akonadi/favoritecollectionsmodel.h>
#include <akonadi/entitylistview.h>
#include <akonadi/itemfetchscope.h>
#include <akonadi/entitymimetypefiltermodel.h>
#include <akonadi/statisticsproxymodel.h>
#include <akonadi/standardactionmanager.h>

// Qt
#include <QDockWidget>
#include <QDBusInterface>
#include <QSplitter>
#include <QGridLayout>
#include <QTimer>
#include <QSignalMapper>
#include <QListView>

// KDE
#include <KAction>
#include <KActionCollection>
#include <KActionMenu>
#include <KApplication>
#include <KFadeWidgetEffect>
#include <KFileDialog>
#include <KVBox>
#include <KMenu>
#include <KMessageBox>
#include <KNotification>
#include <KNotifyConfigWidget>
#include <KSqueezedTextLabel>
#include <kselectionproxymodel.h>
#include <KStatusBar>
#include <KToggleAction>
#include <nepomuk/resourcemanager.h>
#include <kstatusbarofflineindicator.h>
#include <kpimidentities/identity.h>

using namespace Mailody;
using namespace Akonadi;

MainWindow::MainWindow( KXmlGuiWindow *guiWindow, KXMLGUIClient *guiClient,
                        QWidget *parent,
                        KActionCollection* actionCollection ) :
        QWidget( parent ),
        m_xmlGuiWindow( guiWindow ),
        m_xmlGuiClient( guiClient ),
        m_actionCollection( actionCollection ),
        m_standAlone( guiWindow != 0 ),
        m_online( false )
{
    if ( m_xmlGuiWindow )
        m_statusBar = m_xmlGuiWindow->statusBar() ;
    else
        m_statusBar = qobject_cast<KMainWindow*>( parent )->statusBar();

    // First do a check for resources. If not go to the settings dialog...
    QStringList mimeTypeList;
    mimeTypeList << "message/news" << "message/rfc822";
    AgentManager *manager = AgentManager::self();
    bool foundAtLeastOne = false;
    foreach( const Akonadi::AgentInstance& agent, manager->instances() ) {
        foreach( const QString& capable, agent.type().mimeTypes() ) {
            if ( mimeTypeList.contains( capable ) ) {
                foundAtLeastOne = true;
                break;
            }
        }
        if ( foundAtLeastOne )
            break;
    }
    if ( !foundAtLeastOne )
        startupSetup(); // do not refresh mb list now...

    connect( manager,
             SIGNAL( instanceStatusChanged( const Akonadi::AgentInstance & ) ),
             SLOT( slotAgentInstanceStatusChanged( const Akonadi::AgentInstance & ) ) );
    connect( manager,
             SIGNAL( instanceProgressChanged( const Akonadi::AgentInstance & ) ),
             SLOT( slotAgentInstanceProgressChanged( const Akonadi::AgentInstance & ) ) );

    Control::widgetNeedsAkonadi( this );

    // setup the header and the viewer
    m_headerAndViewer = new QSplitter( this );
    m_headerAndViewer->setOrientation( Qt::Vertical );

    QWidget *widg = new QWidget( m_headerAndViewer );
    QGridLayout *gbox = new QGridLayout( widg );
    gbox->setMargin( 0 );
    m_headerWidget = new Mailody::HeaderWidget( this );

    connect( m_headerWidget->view(),
             SIGNAL( actionRequested( MessageBodyWidget::KeyAction ) ),
             SLOT( slotTabAction( MessageBodyWidget::KeyAction ) ) );
    connect( m_headerWidget,
             SIGNAL( selectMessageLeft( const Akonadi::Item& ) ),
             SLOT( slotShowMessage( const Akonadi::Item& ) ) );
    connect( m_headerWidget->view(),
             SIGNAL( selectMessageRight( const Akonadi::Item& ) ),
             SLOT( slotContextMenuHeaders( const Akonadi::Item& ) ) );
    connect( m_headerWidget->view(),
             SIGNAL( selectMessageMiddle( const Akonadi::Item& ) ),
             SLOT( slotMiddleClickInHeaders( const Akonadi::Item& ) ) );

    gbox->addWidget( m_headerWidget, 1, 0 );
    m_right = new KVBox( m_headerAndViewer );

    // create the docks...
    createDocks();

    // create the actions...
    createActions();

    // Tab Widget
    m_tabWidget = new TabWidget( m_right );
    connect( m_tabWidget, SIGNAL( clicked() ), SLOT( slotNewTab() ) );
    connect( m_tabWidget, SIGNAL( currentChanged( int ) ),
             SLOT( slotCurrentTabChanged( int ) ) );
    m_tabWidget->show();

    // Quick Reply Widget
    m_quickReplyWidget = new QuickReplyWidget( m_right );
    connect( m_quickReplyWidget, SIGNAL( finished() ), SLOT( slotQuickReply() ) );
    m_quickReplyWidget->setVisible( false );
    m_quickReplyWidget->setEnabled( false );

    // Statusbar
    if ( m_standAlone ) {
        m_statusText = new KSqueezedTextLabel( this );
        m_statusText->setMargin( 1 );
        m_statusText->setText( i18n( "Ready" ) );
        m_xmlGuiWindow->statusBar()->addWidget( m_statusText,10 );
    }

    m_statusBar->insertItem( i18n( "Quick Reply" ), 2, 2 );
    m_statusBar->insertItem( i18n( "Plain" ), 3, 1 );
    m_statusBar->insertItem( i18nc( "Short for no ext. images",
                                    "No ext." ), 4, 1 );
    connect( m_statusBar, SIGNAL( pressed( int ) ),
             SLOT( slotStatusBarClicked( int ) ) );

    m_statusTimer = new QTimer( this );
    m_statusTimer->setSingleShot( true );
    connect( m_statusTimer, SIGNAL( timeout() ), SLOT( slotStatusBarReady() ) );

    KStatusBarOfflineIndicator *nsi = new KStatusBarOfflineIndicator( this );
    m_statusBar->addWidget( nsi );

    m_systemTray = new SystemTray( this, this );
    connect( m_systemTray, SIGNAL( raise() ), SLOT( raise() ) );

    // Create the layout.
    m_headerAndViewer->show();
    m_folderSplitter = new QSplitter( this );
    QHBoxLayout* box = new QHBoxLayout( this );
    box->addWidget( m_headerAndViewer );

    // Load settings
    loadSettings();

    KConfigGroup configGeneral = KGlobal::config()->group( "General" );
    QString hp = configGeneral.readEntry( "homePage", "http://www.mailody.net" );
    slotOpenURLInNewTab( hp );
    slotToggleHTML();

    setAcceptDrops( false );
    slotSyncAllFolderLists();

    connect( manager, SIGNAL( instanceAdded( const Akonadi::AgentInstance& ) ),
             SLOT( slotInstanceAdded( const Akonadi::AgentInstance & ) ) );
}

MainWindow::~MainWindow()
{
    // If quickReply is open, close it before saving splitter sizes.
    if ( m_quickReplyWidget->isEnabled() ) {
        slotQuickReply();
    }

    KConfigGroup configMain =
        KGlobal::config()->group( "MainWindowSizes" );
    configMain.writeEntry( "hsplitter", m_headerAndViewer->sizes() );
    configMain.writeEntry( "boxsplitter", m_folderSplitter->sizes() );

    KConfigGroup cfg( KGlobal::config(), "CollectionViewState" );
    EntityTreeViewStateSaver saver( m_collectionView );
    saver.saveState( cfg );
    cfg.sync();

    KConfigGroup configToggle = KGlobal::config()->group( "ToggleStates" );
    configToggle.writeEntry( "plaintext", m_plainTextAction->isChecked() );
    configToggle.writeEntry( "source", m_sourceAction->isChecked() );
    configToggle.writeEntry( "fixed", m_fixedfontAction->isChecked() );
    configToggle.writeEntry( "externalimage", m_externalImageAction->isChecked() );
    //configToggle.writeEntry( "sort_youngest", m_childSortAction->isChecked() );
    configToggle.writeEntry( "show_deleted", m_showDeletedAction->isChecked() );
    configToggle.writeEntry( "show_nepomuk", m_nepoAction->isChecked() );
    configToggle.writeEntry( "show_favorites", m_favAction->isChecked() );
    configToggle.writeEntry( "show_collections", m_mbAction->isChecked() );
    configToggle.writeEntry( "favoritesview", m_favoritesView->viewMode() == QListView::IconMode );
    configToggle.sync();
}

void MainWindow::closeEvent( QCloseEvent *event )
{
    // We want to know if there are dirty composers. If that is the case,
    // don't close the mainwindow.
    bool canClose = true;
    QList<Composer*> l = findChildren<Composer*>();
    for ( int i = 0 ; i < l.count(); i++ ) {
        if ( !( l.at( i ) )->canClose() ) {
            event->ignore();
            canClose = false;
        } else
            delete l.at( i );
    }
    canClose ? event->accept() :  event->ignore();
}

void MainWindow::createDocks()
{
    m_collectionView = new Akonadi::EntityTreeView( m_xmlGuiClient, this );
    m_collectionView->setObjectName( "CollectionView" );
    m_collectionView->setSelectionMode( QAbstractItemView::ExtendedSelection );
    connect( m_collectionView, SIGNAL( currentChanged( const Akonadi::Collection ) ),
             SLOT( collectionClicked( const Akonadi::Collection ) ) );

    m_favoritesView = new EntityListView( m_xmlGuiClient, this );
    m_favoritesView->setViewMode( QListView::ListMode );
    connect( m_favoritesView, SIGNAL( clicked( const Akonadi::Collection ) ),
             SLOT( collectionClicked( const Akonadi::Collection ) ) );

    // monitor collection changes
    ChangeRecorder *monitor = new ChangeRecorder( this );
    monitor->setCollectionMonitored( Collection::root() );
    monitor->fetchCollection( true );
    monitor->setAllMonitored( true );
    monitor->setMimeTypeMonitored( "message/rfc822" );
    monitor->itemFetchScope().fetchFullPayload( true );

    Akonadi::EntityTreeModel* collectionModel = new EntityTreeModel( monitor, this );
    collectionModel->setItemPopulationStrategy( EntityTreeModel::LazyPopulation );

    EntityMimeTypeFilterModel *collectionFilter = new EntityMimeTypeFilterModel( this );
    collectionFilter->setSourceModel( collectionModel );
    collectionFilter->addMimeTypeInclusionFilter( Collection::mimeType() );
    collectionFilter->setHeaderGroup( EntityTreeModel::CollectionTreeHeaders );

    StatisticsProxyModel *statisticsProxyModel = new StatisticsProxyModel( this );
    statisticsProxyModel->setToolTipEnabled( true );
    statisticsProxyModel->setExtraColumnsEnabled( false );
    statisticsProxyModel->setSourceModel( collectionFilter );

    QSortFilterProxyModel *sortModel = new QSortFilterProxyModel( this );
    sortModel->setDynamicSortFilter( true );
    sortModel->setSortCaseSensitivity( Qt::CaseInsensitive );
    sortModel->setSourceModel( statisticsProxyModel );

    m_collectionView->setModel( sortModel );

    KSelectionProxyModel *selectionProxyModel = new KSelectionProxyModel( m_collectionView->selectionModel(), this );
    selectionProxyModel->setSourceModel( collectionModel );
    selectionProxyModel->setFilterBehavior( KSelectionProxyModel::ChildrenOfExactSelection );

    FavoriteCollectionsModel *favoritesModel = new FavoriteCollectionsModel( collectionModel, KGlobal::config()->group( "FavoriteCollectionModel" ), this );
    m_favoritesView->setModel( favoritesModel );

    CollectionDelegate* delegate = new CollectionDelegate( m_collectionView );
    m_collectionView->setItemDelegate( delegate );

    StandardActionManager* stdActionManager = new StandardActionManager( m_actionCollection, this );

    //probably needsa lot more, but you have to start somewhere...
    stdActionManager->setActionText( Akonadi::StandardActionManager::CopyItems,
                                     ki18np( "Copy Mail", "Copy %1 Mails" ) );

    stdActionManager->setItemSelectionModel( m_headerWidget->view()->selectionModel() );
    stdActionManager->setCollectionSelectionModel( m_collectionView->selectionModel() );
    stdActionManager->setFavoriteCollectionsModel( favoritesModel );
    stdActionManager->setFavoriteSelectionModel( m_favoritesView->selectionModel() );
    stdActionManager->createAllActions();

    //---------------  Nepomuk  ------------------------------------
    ResourceManager::instance()->init();
    m_nepomuk = new NepomukWidget( this );

    //------------------- Dockers -------------------------------

    // get to the real main window!
    QMainWindow *mainwindow = static_cast<QMainWindow*>( parentWidget()->window() );
    if ( !mainwindow )
        return;

    m_favoriteDock = new QDockWidget( i18n( "Favorites" ), mainwindow );
    m_favoriteDock->setObjectName( "favoriteDock" );
    m_favoriteDock->setWidget( m_favoritesView );
    m_favoriteDock->setFeatures( QDockWidget::DockWidgetMovable |
                                 QDockWidget::DockWidgetClosable );

    mainwindow->addDockWidget( Qt::RightDockWidgetArea, m_favoriteDock );
    m_favAction = m_actionCollection->addAction( "toggle_favdock" );
    m_favAction->setText( i18n( "Show Favorites" ) );
    m_favAction->setCheckable( true );
    m_favAction->setChecked( m_favoriteDock->isVisible() );
    connect( m_favAction, SIGNAL( toggled( bool ) ),
             m_favoriteDock, SLOT( setVisible( bool ) ) );

    m_mailboxDock = new QDockWidget( i18n( "Folders" ), mainwindow );
    m_mailboxDock->setObjectName( "mailboxDock" );
    m_mailboxDock->setWidget( m_collectionView );
    m_mailboxDock->setFeatures( QDockWidget::DockWidgetMovable |
                                QDockWidget::DockWidgetClosable );

    mainwindow->addDockWidget( Qt::RightDockWidgetArea, m_mailboxDock );
    m_mbAction = m_actionCollection->addAction( "toggle_mailboxdock" );
    m_mbAction->setText( i18n( "Show Folders" ) );
    m_mbAction->setCheckable( true );
    m_mbAction->setChecked( m_mailboxDock->isVisible() );
    connect( m_mbAction, SIGNAL( toggled( bool ) ),
             m_mailboxDock, SLOT( setVisible( bool ) ) );

    m_nepoDock = new QDockWidget( i18n( "Tags and Comments" ), mainwindow );
    m_nepoDock->setObjectName( "nepomukDock" );
    m_nepoDock->setWidget( m_nepomuk );
    m_nepoDock->setFeatures( QDockWidget::DockWidgetMovable |
                             QDockWidget::DockWidgetClosable );

    mainwindow->addDockWidget( Qt::RightDockWidgetArea, m_nepoDock );
    m_nepoAction = m_actionCollection->addAction( "toggle_nepodock" );
    m_nepoAction->setText( i18n( "Show Tags and Comments" ) );
    m_nepoAction->setCheckable( true );
    m_nepoAction->setChecked( m_nepoDock->isVisible() );
    connect( m_nepoAction, SIGNAL( toggled( bool ) ),
             m_nepoDock, SLOT( setVisible( bool ) ) );
}

void MainWindow::loadSettings()
{
    KConfigGroup configMain = KGlobal::config()->group( "MainWindowSizes" );
    QList<int> defaultsize2 = configMain.readEntry( "hsplitter", QList<int>() );
    if ( defaultsize2.isEmpty() ) {
        defaultsize2.append( 300 );
        defaultsize2.append( 400 );
    }
    m_headerAndViewer->setSizes( defaultsize2 );

    QList<int> defaultsize3 = configMain.readEntry( "boxsplitter", QList<int>() );
    if ( defaultsize3.isEmpty() ) {
        defaultsize3.append( 200 );
        defaultsize3.append( 600 );
    }
    m_folderSplitter->setSizes( defaultsize3 );

    const KConfigGroup cfg( KGlobal::config(), "CollectionViewState" );
    EntityTreeViewStateSaver* saver = new EntityTreeViewStateSaver( m_collectionView );
    saver->restoreState( cfg );

    KConfigGroup configToggle = KGlobal::config()->group( "ToggleStates" );
    m_plainTextAction->setChecked(
        configToggle.readEntry( "plaintext", true ) );
    m_sourceAction->setChecked(
        configToggle.readEntry( "source", false ) );
    m_fixedfontAction->setChecked(
        configToggle.readEntry( "fixed", false ) );
    m_externalImageAction->setChecked(
        configToggle.readEntry( "externalimage", false ) );
    //m_childSortAction->setChecked(
    //    configToggle.readEntry( "sort_youngest", false ) );
    m_showDeletedAction->setChecked(
        configToggle.readEntry( "show_deleted", false ) );
    m_nepoAction->setChecked(
        configToggle.readEntry( "show_nepomuk", true ) );
    m_favAction->setChecked(
        configToggle.readEntry( "show_favorites", true ) );
    m_mbAction->setChecked(
        configToggle.readEntry( "show_collections", true ) );

    bool iconmode = configToggle.readEntry( "favoritesview", true );
    iconmode = true; //TODO: crashes on ListMode...
    m_favoritesView->setViewMode( iconmode ? QListView::IconMode : QListView::ListMode );
    m_favViewAction->setChecked( iconmode );

    slotHideDeleted();
}

void MainWindow::createActions()
{
    m_nextUnreadAction = new KAction( this );
    m_nextUnreadAction->setText( i18n( "Next Unread" ) );
    m_nextUnreadAction->setIcon( KIcon( "go-down-search" ) );
    m_nextUnreadAction->setShortcut( Qt::Key_Space );
    m_actionCollection->addAction( "next_unread", m_nextUnreadAction );
    connect( m_nextUnreadAction, SIGNAL( triggered( bool ) ),
             SLOT( slotNextUnread() ) );

    m_selectAllAction = new KAction( this );
    m_selectAllAction->setText( i18n( "Select All Messages" ) );
    m_selectAllAction->setIcon( KIcon( "edit-select-all" ) );
    m_selectAllAction->setShortcut( KStandardShortcut::selectAll() );
    m_actionCollection->addAction( "select_all_msg", m_selectAllAction );
    connect( m_selectAllAction, SIGNAL( triggered( bool ) ),
             SLOT( slotSelectAllMessages() ) );

    // TODO: where are my threads??
    //m_childSortAction = new KToggleAction( this );
    //m_childSortAction->setText( i18n( "Sort on Youngest Child" ) );
    //m_childSortAction->setIcon( KIcon( "view-sort-ascending" ) );
    //actionCollection->addAction( "sort_youngest_child", m_childSortAction );
    //connect( m_childSortAction, SIGNAL( triggered( bool ) ),
    //       SLOT( slotSortOnYoungestChild() ) );

    m_showDeletedAction = new KToggleAction( this );
    m_showDeletedAction->setText( i18n( "Show Deleted Messages" ) );
    m_actionCollection->addAction( "show_deleted", m_showDeletedAction );
    connect( m_showDeletedAction, SIGNAL( triggered( bool ) ),
             SLOT( slotHideDeleted() ) );

    KAction* action = m_actionCollection->addAction( "options_configure" );
    action->setIcon( KIcon( "configure" ) );
    action->setText( i18n( "&Configure Mailody..." ) );
    connect( action, SIGNAL( triggered( bool ) ), SLOT( slotSetup() ) );

    KStandardAction::configureNotifications( this,
            SLOT( slotNotifications() ), m_actionCollection );

    KAction *quit = new KAction( this );
    quit->setText( i18n( "Quit" ) );
    quit->setIcon( KIcon( "application-exit" ) );
    quit->setShortcut( KStandardShortcut::quit() );
    m_actionCollection->addAction( "mailody_quit", quit );
    connect( quit, SIGNAL( triggered( bool ) ),
             SLOT( slotQuit() ) );

    KAction *sync_all = new KAction( this );
    sync_all->setText( i18n( "Sync Folder List (all accounts)" ) );
    m_actionCollection->addAction( "sync_mblists", sync_all );
    connect( sync_all, SIGNAL( triggered( bool ) ),
             SLOT( slotSyncAllFolderLists() ) );

    m_syncResource = new KAction( this );
    m_syncResource->setText( i18n( "Sync Folder List (this account)" ) );
    m_actionCollection->addAction( "sync_mblist", m_syncResource );
    connect( m_syncResource, SIGNAL( triggered( bool ) ),
             SLOT( slotSyncFolderList() ) );

    KAction *getmail = new KAction( this );
    getmail->setText( i18n( "Check for Mail Now" ) );
    getmail->setShortcut( Qt::CTRL + Qt::Key_L );
    getmail->setIcon( KIcon( "mail-receive" ) );
    m_actionCollection->addAction( "check_mail", getmail );
    connect( getmail, SIGNAL( triggered( bool ) ),
             SLOT( slotCheckMail() ) );

    m_msgSaveAction = new KAction( this );
    m_msgSaveAction->setText( i18n( "Save as..." ) );
    m_msgSaveAction->setShortcut( KStandardShortcut::save() );
    m_msgSaveAction->setIcon( KIcon( "document-save-as" ) );
    m_actionCollection->addAction( "mailody_save", m_msgSaveAction );
    connect( m_msgSaveAction, SIGNAL( triggered( bool ) ),
             SLOT( slotSave() ) );

    m_msgPrintAction = new KAction( this );
    m_msgPrintAction->setText( i18n( "Print..." ) );
    m_msgPrintAction->setShortcut( KStandardShortcut::print() );
    m_msgPrintAction->setIcon( KIcon( "document-print" ) );
    m_actionCollection->addAction( "mailody_print", m_msgPrintAction );
    connect( m_msgPrintAction, SIGNAL( triggered( bool ) ),
             SLOT( slotPrint() ) );

#ifdef WITHNEPOMUMPLAYGROUND
    m_msgPossibleTagAction = new KAction( this );
    m_msgPossibleTagAction->setText( i18n( "Possible tags..." ) );
    m_actionCollection->addAction( "mailody_possible_tags", m_msgPossibleTagAction );
    m_msgPossibleTagAction->setIcon( KIcon( "nepomuk" ) );
    connect( m_msgPossibleTagAction, SIGNAL( triggered( bool ) ),
             SLOT( slotShowPossibleTags() ) );
#endif

    m_msgZoomInAction = new KAction( this );
    m_msgZoomInAction->setText( i18n( "Zoom In" ) );
    m_msgZoomInAction->setShortcut( KStandardShortcut::zoomIn() );
    m_msgZoomInAction->setIcon( KIcon( "zoom-in" ) );
    m_actionCollection->addAction( "mailody_zoom_in", m_msgZoomInAction );
    connect( m_msgZoomInAction, SIGNAL( triggered( bool ) ),
             SLOT( slotZoomIn() ) );

    m_msgZoomOutAction = new KAction( this );
    m_msgZoomOutAction->setText( i18n( "Zoom Out" ) );
    m_msgZoomOutAction->setShortcut( KStandardShortcut::zoomOut() );
    m_msgZoomOutAction->setIcon( KIcon( "zoom-out" ) );
    m_actionCollection->addAction( "mailody_zoom_out", m_msgZoomOutAction );
    connect( m_msgZoomOutAction, SIGNAL( triggered( bool ) ),
             SLOT( slotZoomOut() ) );

    m_plainTextAction = new KToggleAction( this );
    m_plainTextAction->setText( i18n( "Show as Text" ) );
    m_plainTextAction->setShortcut( Qt::Key_F10 );
    m_actionCollection->addAction( "toggle_html", m_plainTextAction );
    connect( m_plainTextAction, SIGNAL( triggered( bool ) ),
             SLOT( slotToggleHTML() ) );

    m_sourceAction = new KToggleAction( this );
    m_sourceAction->setText( i18n( "Show Source" ) );
    m_sourceAction->setShortcut( Qt::Key_V );
    m_actionCollection->addAction( "toggle_source", m_sourceAction );
    connect( m_sourceAction, SIGNAL( triggered( bool ) ),
             SLOT( slotToggleSource() ) );

    m_fixedfontAction = new KToggleAction( this );
    m_fixedfontAction->setText( i18n( "Use Fixed Font" ) );
    m_fixedfontAction->setShortcut( Qt::Key_X );
    m_actionCollection->addAction( "toggle_fixedfont", m_fixedfontAction );
    connect( m_fixedfontAction, SIGNAL( triggered( bool ) ),
             SLOT( slotToggleFixedFont() ) );

    m_externalImageAction = new KToggleAction( this );
    m_externalImageAction->setText( i18n( "Load External Images" ) );
    m_actionCollection->addAction( "toggle_externalimage", m_externalImageAction );
    connect( m_externalImageAction, SIGNAL( triggered( bool ) ),
             SLOT( slotToggleExternalImage() ) );

    m_msgDelAction = new KToggleAction( this );
    m_msgDelAction->setText( i18n( "Message Deleted" ) );
    m_msgDelAction->setIcon( KIcon( "edit-delete" ) );
    m_msgDelAction->setShortcut( Qt::Key_Delete );
    m_actionCollection->addAction( "delete_message", m_msgDelAction );
    connect( m_msgDelAction, SIGNAL( triggered( bool ) ),
             SLOT( slotToggleDelete() ) );

    m_purgeAction = new KAction( this );
    m_purgeAction->setText( i18n( "Remove Deleted Messages" ) );
    m_purgeAction->setIcon( KIcon( "trash-empty" ) );
    m_actionCollection->addAction( "purge_collection", m_purgeAction );
    connect( m_purgeAction, SIGNAL( triggered( bool ) ),
             SLOT( slotPurge() ) );

    m_msgNewAction = new KAction( this );
    m_msgNewAction->setText( i18n( "New" ) );
    m_msgNewAction->setIcon( KIcon( "mail-message-new" ) );
    m_msgNewAction->setShortcut( KStandardShortcut::openNew() );
    m_msgNewAction->setEnabled( true );
    m_actionCollection->addAction( "compose_new_message", m_msgNewAction );
    connect( m_msgNewAction, SIGNAL( triggered( bool ) ),
             SLOT( slotNewMessage() ) );

    m_msgNewActionMenu = new KActionMenu( i18n( "New" ), 0 );
    m_msgNewActionMenu->setIcon( KIcon( "mail-message-new" ) );
    m_msgNewActionMenu->setEnabled( true );
    m_actionCollection->addAction( "compose_new_messageMenu", m_msgNewActionMenu );

    connect( m_msgNewActionMenu,
             SIGNAL( triggered( Qt::MouseButtons,Qt::KeyboardModifiers ) ),
             SLOT( slotNewMessage() ) );
    connect( m_msgNewActionMenu->menu(), SIGNAL( aboutToShow() ),
             SLOT( slotNewMessageMenu() ) );
    connect( m_msgNewActionMenu->menu(), SIGNAL( triggered( QAction* ) ),
             SLOT( slotNewMessageMenuSelected( QAction* ) ) );

    m_msgReplyAction = new KAction( this );
    m_msgReplyAction->setText( i18n( "Reply" ) );
    m_msgReplyAction->setIcon( KIcon( "mail-reply-all" ) );
    m_msgReplyAction->setShortcut( Qt::CTRL + Qt::Key_R );
    m_actionCollection->addAction( "reply_message", m_msgReplyAction );
    connect( m_msgReplyAction, SIGNAL( triggered( bool ) ),
             SLOT( slotReplyMessage() ) );

    m_msgReEditAction = new KAction( this );
    m_msgReEditAction->setText( i18n( "Re-Edit" ) );
    m_msgReEditAction->setIcon( KIcon( "document-properties" ) );
    m_msgReEditAction->setShortcut( Qt::CTRL + Qt::Key_Y );
    m_actionCollection->addAction( "reedit_message", m_msgReEditAction );
    connect( m_msgReEditAction, SIGNAL( triggered( bool ) ),
             SLOT( slotReEditMessage() ) );

    m_msgForwardAction = new KAction( this );
    m_msgForwardAction->setText( i18nc( "Forwarding a message", "Forward" ) );
    m_msgForwardAction->setIcon( KIcon( "mail-forward" ) );
    m_msgForwardAction->setShortcut( Qt::CTRL + Qt::Key_F );
    m_actionCollection->addAction( "forward_message", m_msgForwardAction );
    connect( m_msgForwardAction, SIGNAL( triggered( bool ) ),
             SLOT( slotForwardMessage() ) );
    m_msgForwardActionMenu = new KActionMenu( this );
    m_msgForwardActionMenu->setText( i18nc( "Forwarding a message", "Forward" ) );
    m_msgForwardActionMenu->setIcon( KIcon( "mail-forward" ) );
    m_actionCollection->addAction( "forward_messageMenu", m_msgForwardActionMenu );

    connect( m_msgForwardActionMenu,
             SIGNAL( triggered( Qt::MouseButtons,Qt::KeyboardModifiers ) ),
             SLOT( slotForwardMessage() ) );
    connect( m_msgForwardActionMenu->menu(), SIGNAL( aboutToShow() ),
             SLOT( slotForwardMessageMenu() ) );
    connect( m_msgForwardActionMenu->menu(), SIGNAL( triggered( QAction* ) ),
             SLOT( slotForwardMessageMenuSelected( QAction* ) ) );

    QSignalMapper* signalMapper = new QSignalMapper( this );

    m_msgUser0Action = new KAction( this );
    m_msgUser0Action->setText( i18n( "No Label" ) );
    m_msgUser0Action->setShortcut( Qt::Key_0 );
    m_actionCollection->addAction( "tag_0", m_msgUser0Action );
    connect( m_msgUser0Action, SIGNAL( triggered( bool ) ),
             signalMapper, SLOT( map() ) );
    signalMapper->setMapping( m_msgUser0Action, 0 );

    m_msgUser1Action = new KAction( this );
    m_msgUser1Action->setText( i18n( "Label &1" ) );
    m_msgUser1Action->setIcon( KIcon( "label1" ) );
    m_msgUser1Action->setShortcut( Qt::Key_1 );
    m_actionCollection->addAction( "tag_1", m_msgUser1Action );
    connect( m_msgUser1Action, SIGNAL( triggered( bool ) ),
             signalMapper, SLOT( map() ) );
    signalMapper->setMapping( m_msgUser1Action, 1 );

    m_msgUser2Action = new KAction( this );
    m_msgUser2Action->setText( i18n( "Label &2" ) );
    m_msgUser2Action->setIcon( KIcon( "label2" ) );
    m_msgUser2Action->setShortcut( Qt::Key_2 );
    m_actionCollection->addAction( "tag_2", m_msgUser2Action );
    connect( m_msgUser2Action, SIGNAL( triggered( bool ) ),
             signalMapper, SLOT( map() ) );
    signalMapper->setMapping( m_msgUser2Action, 2 );

    m_msgUser3Action = new KAction( this );
    m_msgUser3Action->setText( i18n( "Label &3" ) );
    m_msgUser3Action->setIcon( KIcon( "label3" ) );
    m_msgUser3Action->setShortcut( Qt::Key_3 );
    m_actionCollection->addAction( "tag_3", m_msgUser3Action );
    connect( m_msgUser3Action, SIGNAL( triggered( bool ) ),
             signalMapper, SLOT( map() ) );
    signalMapper->setMapping( m_msgUser3Action, 3 );

    m_msgUser4Action = new KAction( this );
    m_msgUser4Action->setText( i18n( "Label &4" ) );
    m_msgUser4Action->setIcon( KIcon( "label4" ) );
    m_msgUser4Action->setShortcut( Qt::Key_4 );
    m_actionCollection->addAction( "tag_4", m_msgUser4Action );
    connect( m_msgUser4Action, SIGNAL( triggered( bool ) ),
             signalMapper, SLOT( map() ) );
    signalMapper->setMapping( m_msgUser4Action, 4 );

    m_msgUser5Action = new KAction( this );
    m_msgUser5Action->setText( i18n( "Label &5" ) );
    m_msgUser5Action->setIcon( KIcon( "label5" ) );
    m_msgUser5Action->setShortcut( Qt::Key_5 );
    m_actionCollection->addAction( "tag_5", m_msgUser5Action );
    connect( m_msgUser5Action, SIGNAL( triggered( bool ) ),
             signalMapper, SLOT( map() ) );
    signalMapper->setMapping( m_msgUser5Action, 5 );

    connect( signalMapper, SIGNAL( mapped( int ) ),
             this, SLOT( slotUserFlag( int ) ) );

    m_favViewAction = new KToggleAction( this );
    m_favViewAction->setText( i18n( "Use Icons in Favorites" ) );
    m_actionCollection->addAction( "change_favview", m_favViewAction );
    connect( m_favViewAction, SIGNAL( triggered( bool ) ),
             SLOT( slotChangeFavoritesView() ) );

    m_closeAction = new KAction( this );
    m_closeAction->setText( i18n( "Close Tab" ) );
    m_closeAction->setShortcut( Qt::CTRL + Qt::Key_W );
    m_actionCollection->addAction( "close_tab", m_closeAction );
    connect( m_closeAction, SIGNAL( triggered( bool ) ),
             SLOT( slotDeleteTab() ) );
}

void MainWindow::slotSyncAllFolderLists()
{
    // Mimetypes we support.
    QStringList mimeTypeList;
    mimeTypeList << "message/news" << "message/rfc822";

    AgentManager *manager = AgentManager::self();
    // not const & as synchronizeCollectionTree() is not const.
    foreach( Akonadi::AgentInstance agent, manager->instances() ) { // krazy:exclude=foreach
        foreach( const QString& capable, agent.type().mimeTypes() ) {
            if ( mimeTypeList.contains( capable ) ) {

                if ( agent.type().identifier() == "akonadi_imap_resource" ) {

                    // set to manual expunge
                    QString dbusstr = "org.freedesktop.Akonadi.Resource.";
                    dbusstr.append( agent.identifier() );
                    QDBusInterface dbus( dbusstr, "/Settings" );
                    dbus.call( QLatin1String( "setAutomaticExpungeEnabled" ),
                               false );

                    agent.reconfigure();
                    kDebug() << "Set no automexpunge on: " << agent.name() << agent.identifier();
                }

                agent.synchronizeCollectionTree();
            }
        }
    }
}

void MainWindow::slotInstanceAdded( const Akonadi::AgentInstance &agent )
{
    kDebug() << "new resource.";

    if ( agent.type().identifier() == "akonadi_imap_resource" ) {

        // set to manual expunge
        QString dbusstr = "org.freedesktop.Akonadi.Resource.";
        dbusstr.append( agent.identifier() );
        QDBusInterface dbus( dbusstr, "/Settings" );
        dbus.call( QLatin1String( "setAutomaticExpungeEnabled" ),
                   false );

        agent.reconfigure();

        kDebug() << "Set no automexpunge on: " << agent.name();
    }

    //syncCollectionTree() is not const....
    AgentInstance agent2 = agent;
    agent2.synchronizeCollectionTree();
}

void MainWindow::slotSyncFolderList()
{
    const QString identifier = m_currentCollection.resource();
    Akonadi::AgentInstance agent = Akonadi::AgentManager::self()->instance( identifier );
    if ( agent.isValid() )
        agent.synchronizeCollectionTree();
}

Akonadi::Collection MainWindow::findAnInbox()
{



    //todo: PORT to specialfolder, when API review has been committed for this. part






    // kDebug() << "Trying to find inbox";

    // first iterate through the resources.
    int totalResources = m_collectionView->model()->rowCount();
    for ( int i = 0; i < totalResources ; ++i ) {
        const QModelIndex resourceIndex = m_collectionView->model()->index( i, 0 );
        // kDebug() << "Evaluating resource:" << m_collectionView->model()->data( resourceIndex );

        // then iterate through the top level folders of that resource.
        int totalFolders = m_collectionView->model()->rowCount( resourceIndex );
        // kDebug() << "We have " << totalFolders << " folders in there";
        for ( int j = 0 ; j < totalFolders ; ++j ) {
            const QModelIndex folderIndex = m_collectionView->model()->index( j, 0, resourceIndex );
            const Akonadi::Collection collection =
                m_collectionView->model()->data( folderIndex,
                                                 Akonadi::EntityTreeModel::CollectionRole ).value<Akonadi::Collection>();
            // kDebug() << "Found folder: " << collection.remoteId();

            // see if we have the inbox.
            if ( collection.remoteId().toLower() == "inbox" ) {
                return collection;
            }
        }
    }

    KMessageBox::information(
        this,
        i18n( "There is no Inbox found in any resource. Starting a new message "
              "will cause the message to be lost after you have sent it. You will "
              "not have a local copy anymore. If you want a copy, one way to do "
              "this is to add yourself as a CC to the message." ),
        i18n( "Cannot save the message" ),
        "IWantNoLocalCopyWhenNoInboxIsFound" );
    return Akonadi::Collection();
}

void MainWindow::slotOpenComposer( const QString& address,
                                   const QString& files,
                                   const QString& subject,
                                   const QString& body,
                                   Akonadi::Collection mb )
{
    // kDebug() << address << files << subject << body;

    Composer *composer = new Composer( this, mb );
    connect( composer, SIGNAL( showSettings() ), SLOT( slotSetup() ) );

    // emails
    const QStringList addrList = address.split( ',',  QString::SkipEmptyParts );
    for ( QStringList::ConstIterator it = addrList.constBegin();
            it != addrList.constEnd();
            ++it ) {
        QString a = *it;
        if ( a.startsWith( QLatin1String( "mailto:" ), Qt::CaseInsensitive ) )
            a.remove( 0,7 );
        a.replace( "%40","@" );

        // kDebug() << "Composer requested: " << a;
        if ( !a.isEmpty() )
            composer->setRcpt( a, RecipientWidget::To );
    }

    // attachments
    const QStringList fileList =  files.split( ";;", QString::SkipEmptyParts );
    for ( QStringList::ConstIterator it = fileList.constBegin();
            it != fileList.constEnd();
            ++it ) {
        const KUrl url = KUrl( *it );
        if (( url ).isValid() )
            composer->addAttachment( url );
    }
    composer->setSubject( subject );
    composer->setMsg( body );
    composer->show();
}

//---------------------- message --------------------------------//

void MainWindow::setTag( const Akonadi::Item& item, const QString& tag )
{
    Q_ASSERT( item.isValid() );
    // kDebug() << item.id() << tag;
    FlagModifyJob *job = new FlagModifyJob( item, tag, this );
    connect( job, SIGNAL( result( KJob* ) ), SLOT( slotResultStoreJob( KJob* ) ) );
}

void MainWindow::setTag( const QString& tag )
{
    // kDebug() << tag;
    QList< Akonadi::Item > list = m_headerWidget->currentlySelectedItems();
    foreach( const Akonadi::Item &item, list ) {
        setTag( item, tag );
    }
}

void MainWindow::removeSingleTag( Akonadi::Item item, const QString& flag )
{
    item.clearFlag( flag.toLatin1() );
    Akonadi::ItemModifyJob *job = new Akonadi::ItemModifyJob( item );
    connect( job, SIGNAL( result( KJob* ) ), SLOT( slotResultStoreJob( KJob* ) ) );
}

void MainWindow::removeTag( const QString& tag )
{
    // kDebug() << tag;
    QList< Akonadi::Item > list = m_headerWidget->currentlySelectedItems();
    foreach( const Akonadi::Item &item, list ) {
        removeSingleTag( item, tag );
    }
}

void MainWindow::slotToggleDelete()
{
    // kDebug();
    if ( !m_msgDelAction->isChecked() ) {
        removeTag( "\\Deleted" );
    } else {
        setTag( "\\Deleted" );
        m_headerWidget->view()->next();

        // hide deleted...
        if ( !m_showDeletedAction->isChecked() )
            QTimer::singleShot( 0,this, SLOT( slotHideDeleted() ) );
    }
}

void MainWindow::slotUserFlag( int flag )
{
    setTag( "$label" + QString::number( flag ) );
}

void MainWindow::slotQuickReply()
{
    static QList<int> s_splitterSizes;

    if ( m_quickReplyWidget->isEnabled() ) {
        KFadeWidgetEffect *animation = new KFadeWidgetEffect( m_headerAndViewer );
        m_headerAndViewer->setSizes( s_splitterSizes );
        m_quickReplyWidget->setEnabled( false );
        m_quickReplyWidget->setVisible( false );
        animation->start();
        return;
    }

    // Save the current layout.
    s_splitterSizes = m_headerAndViewer->sizes();

    // Initialise the widget
    if ( m_tabWidget->isCurrentTab( TabWidget::Browser ) )
        return;
    MessageWidget *i = m_tabWidget->currentTabWidget();
    m_quickReplyWidget->setMessage( m_messageDataList.value( i ), m_currentCollection );

    // Make the headerlist 0 high and make the widget visible.
    KFadeWidgetEffect *animation = new KFadeWidgetEffect( m_headerAndViewer );
    QList<int> newSplitterSizes;
    newSplitterSizes << 0 << s_splitterSizes.at( 1 );
    m_headerAndViewer->setSizes( newSplitterSizes );
    m_quickReplyWidget->setEnabled( true );
    m_quickReplyWidget->setVisible( true );
    animation->start();
}

void MainWindow::slotToggleHTML()
{
    m_tabWidget->currentTabWidget()->body()->showHTML( !m_plainTextAction->isChecked() );
    m_fixedfontAction->setEnabled( m_plainTextAction->isChecked() );
    setStatusBarStates();
}

void MainWindow::slotToggleSource()
{
    MessageWidget *i = m_tabWidget->currentTabWidget();
    m_sourceAction->isChecked() ? i->body()->setViewSource()
    :i->body()->showHTML( !m_plainTextAction->isChecked() );
    m_plainTextAction->setEnabled( !m_sourceAction->isChecked() );
}

void MainWindow::slotToggleFixedFont()
{
    MessageWidget *i = m_tabWidget->currentTabWidget();
    i->body()->setFixedFont( m_fixedfontAction->isChecked() );
}

void MainWindow::slotToggleExternalImage()
{
    MessageWidget *tv = m_tabWidget->currentTabWidget();
    if ( m_externalImageAction->isChecked() ) {
        int i = KMessageBox::warningContinueCancel( this,
                i18n( "Loading external images gives spammers the "
                      "acknowledgement that you received this message "
                      "so they will use your email address to spam you. "
                      "So you should only continue for very trusted messages." ),
                QString(),
                KStandardGuiItem::cont(),
                KStandardGuiItem::cancel(),
                "IKnowRisksOfExternalImages" );
        if ( i == KMessageBox::Continue )
            tv->body()->setExternalImage( true );
        else
            m_externalImageAction->setChecked( false );
    } else
        tv->body()->setExternalImage( false );

    setStatusBarStates();
}

void MainWindow::slotShowMessage( const Akonadi::Item& item )
{
    showMessage( item, m_currentCollection );
}

void MainWindow::slotMiddleClickInHeaders( const Akonadi::Item& item )
{
    Q_ASSERT( item.hasPayload() );
    slotNewTab();
    showMessage( item, m_currentCollection );
}

void MainWindow::showMessage( const Akonadi::Item &item, const Akonadi::Collection &collection )
{
    if ( !item.hasPayload() )
        return;

    // create the messageData.
    MessageData* messageData = new MessageData( this );
    messageData->setMsg( item.payload<KMime::Message::Ptr>().get() );

    // setup the tab, disable external images.
    m_tabWidget->setCurrentTabType( TabWidget::Message );
    MessageWidget *tv = m_tabWidget->currentTabWidget();
    m_externalImageAction->setChecked( false );
    tv->body()->setExternalImage( false, false );

    // store the messagedata belonging to the tab.
    m_messageDataList[ tv ] = messageData;
    m_itemList[ tv ] = item;

    // Update the tab title.
    int i = m_tabWidget->indexOf( tv );
    m_tabWidget->setTabText( i, messageData->sender() );

    // set the message to the tab and the nepomuk widget
    tv->setMsg( messageData );
    m_nepomuk->setMsg( item.url() );

    // If quickReply is activated, set the message.
    if ( m_quickReplyWidget->isEnabled() ) {
        m_quickReplyWidget->setMessage( messageData, collection );
    }

    setActionStates();

    const QByteArray seen( "\\Seen" );
    if ( !item.hasFlag( seen ) ) {
        Akonadi::Item modifiedItem( item );
        modifiedItem.setFlag( seen );
        new Akonadi::ItemModifyJob( modifiedItem );
    }
}

void MainWindow::slotSelectAllMessages()
{
    m_headerWidget->view()->selectAll();
}

void MainWindow::slotSortOnYoungestChild()
{
    // m_headerWidget->setSortOnChild( m_childSortAction->isChecked() );
}

void MainWindow::slotHideDeleted()
{
    m_headerWidget->setHideDeleted( !m_showDeletedAction->isChecked() );
}

void MainWindow::slotNextUnread()
{
    //expand this to search multiple collections.
    const bool ok = m_headerWidget->view()->nextUnread();
    if ( !ok )
        kDebug() << "Nothing new found";
}

//------------------------ Tabs --------------------------------//
void MainWindow::slotNewTab()
{
    MessageWidget *widget = new MessageWidget( m_tabWidget );
    m_tabWidget->addMessageTab( widget, i18n( "Message" ) );
    widget->body()->showHTML( !m_plainTextAction->isChecked() );
    if ( m_sourceAction->isChecked() )
        widget->body()->setViewSource();
    widget->body()->setFixedFont( m_fixedfontAction->isChecked() );
    widget->body()->setExternalImage( m_externalImageAction->isChecked() );

    connect( widget->body(), SIGNAL( status( const QString& ) ),
             SLOT( slotSimpleMessageStatusBar( const QString& ) ) );
    connect( widget->body(), SIGNAL( openInTab( const QString& ) ),
             SLOT( slotOpenURLInNewTab( const QString& ) ) );
    connect( widget->body(), SIGNAL( openComposer( const QString& ) ),
             SLOT( slotOpenComposer( const QString& ) ) );
    connect( widget->body(), SIGNAL( pageTitle( const QString& ) ),
             SLOT( slotUpdateTabTitle( const QString& ) ) );
    connect( widget->headers(),
             SIGNAL( openComposer( const QString& ) ),
             SLOT( slotOpenComposer( const QString& ) ) );
    connect( widget, SIGNAL( possibleTagsAvailable( bool ) ),
             SLOT( slotEnablePossibleTagsAction( bool ) ) );
}

void MainWindow::slotDeleteTab()
{
    m_tabWidget->slotDeleteCurrentWidget();
    slotCurrentTabChanged( m_tabWidget->currentIndex() );
}

void MainWindow::slotCurrentTabChanged( int i )
{
    Q_UNUSED( i );
    /* TODO: Handle the the reselection the Collection and
     * item in the headerview
     *
    bool message = m_tabWidget->currentWidgetType( TabWidget::Message );
    MessageWidget *tb;
    if ( message )
        tb = m_tabWidget->currentTabWidget();

    m_mailboxList->blockSignals(true);
    slotSelectMB(tb->msg()->mb());
    m_mailboxList->blockSignals(false);
    */
    setActionStates();
}

void MainWindow::slotTabAction( MessageBodyWidget::KeyAction action )
{
    m_tabWidget->currentTabWidget()->body()->action( action );
}

void MainWindow::slotZoomIn()
{
    m_tabWidget->currentTabWidget()->body()->action( MessageBodyWidget::ZoomIn );
}

void MainWindow::slotZoomOut()
{
    m_tabWidget->currentTabWidget()->body()->action( MessageBodyWidget::ZoomOut );
}

//---------------------- Menus ---------------------------------//

void MainWindow::slotContextMenuHeaders( const Akonadi::Item& )
{
    QMenu *p = new QMenu( this );

    p->addAction( m_msgReplyAction );
    p->addAction( m_msgForwardAction );
    p->addAction( m_msgReEditAction );
    p->addAction( m_msgDelAction );
    p->addSeparator();
    p->addAction( m_msgUser0Action );
    p->addAction( m_msgUser1Action );
    p->addAction( m_msgUser2Action );
    p->addAction( m_msgUser3Action );
    p->addAction( m_msgUser4Action );
    p->addAction( m_msgUser5Action );
    p->popup( QCursor::pos() );
}

void MainWindow::slotChangeFavoritesView()
{
    if ( m_favoritesView->viewMode() == QListView::ListMode ) {
        m_favoritesView->setViewMode( QListView::IconMode );
    } else {
        m_favoritesView->setViewMode( QListView::ListMode );
    }
}

//------------------------- Setup & Quit --------------------------//

void MainWindow::startupSetup()
{
    QPointer<Setup> setup = new Setup( this );
    setup->exec();
    delete setup;
}

void MainWindow::slotSetup()
{
    // kDebug();
    QPointer<Setup> setup = new Setup( this );
    if ( setup->exec() != QDialog::Accepted ) {
        delete setup;
        return;
    } else
        delete setup;

    // For the composer, to know when the settings are changed and a new delivery can be made if
    // needed.
    emit setupFinished();

    // Maybe there was a change in the auto saving attachment, so notify the tabs...
    for ( int o = 0; o < m_tabWidget->count() ; ++o )
        m_tabWidget->tabWidget( o )->autoSaveAttachments() ;

    slotSyncFolderList();
}

void MainWindow::slotNewMessage( const QString &to )
{
    slotOpenComposer( to, QString(), QString(), QString(), m_currentCollection );
}

void MainWindow::slotNewMessageMenu()
{
    messageMenu( m_msgNewActionMenu );
}

void MainWindow::slotNewMessageMenuSelected( QAction *i )
{
    slotNewMessage( i->text() );
}

void MainWindow::slotReplyMessage()
{
    MessageWidget *i =  m_tabWidget->currentTabWidget();

    MessageData *msg = i->msg();

    Composer *composer = new Composer( this, m_currentCollection );
    if ( !msg->replyTo().isEmpty() ) {
        if ( !Global::myEmail( msg->replyTo() ) )
            composer->setRcpt( msg->replyTo(), RecipientWidget::To );
    } else {
        if ( !Global::myEmail( msg->sender() ) )
            composer->setRcpt( msg->sender_full( false /* no company*/ ),
                               RecipientWidget::To );
    }

    const QStringList list = msg->to_list();
    if ( !list.isEmpty() ) {
        QStringList::ConstIterator it=list.constBegin();
        for ( ; it!=list.constEnd(); ++it )
            if ( !Global::myEmail(( *it ) ) )
                composer->setRcpt(( *it ), RecipientWidget::To );
            else
                composer->setIdentity( Global::getIdentityForEmail( *it ) );
    }

    const QStringList cclist = msg->cc_list();
    if ( !cclist.isEmpty() ) {
        QStringList::ConstIterator it=cclist.constBegin();
        for ( ; it!=cclist.constEnd(); ++it )
            if ( !Global::myEmail(( *it ) ) )
                composer->setRcpt(( *it ), RecipientWidget::Cc );
            else
                composer->setIdentity( Global::getIdentityForEmail( *it ) );
    }

    composer->inReplyTo( msg->messageID() );

    if ( msg->subject().startsWith( QLatin1String( "Re:" ), Qt::CaseInsensitive ) )
        composer->setSubject( msg->subject() );
    else
        composer->setSubject( QLatin1String( "Re: " ) + msg->subject() );

    QString toQuote;
    if ( !i->body()->selectedText().isEmpty() )
        toQuote = i->body()->selectedText();
    else
        toQuote = msg->raw();

    composer->setMsgAndQuote( toQuote, msg->body() );
    composer->show();
}

void MainWindow::slotForwardMessage( const QString &to )
{
    MessageWidget *i = m_tabWidget->currentTabWidget();

    MessageData *msg = i->msg();
    Composer *composer = new Composer( this, m_currentCollection );
    composer->setSubject( "Fwd: " + msg->subject() );

    QString toQuote;
    if ( !i->body()->page()->selectedText().isEmpty() )
        toQuote = i->body()->page()->selectedText();
    else
        toQuote = msg->raw();

    const QString text = i18n( "At %1, %2 wrote:\n",
                               KGlobal::locale()->formatDateTime( msg->date(),
                                                                  KLocale::LongDate ),
                               msg->sender_full( false /* no company */ ) )
                         + Global::quote( toQuote, false /*leave sig in there*/ ) + '\n';
    composer->setMsg( text );

    if ( !to.isEmpty() )
        composer->setRcpt( to, RecipientWidget::To );

    // try to find the right identitiy to forward with...
    const QStringList list = msg->to_list();
    if ( !list.isEmpty() ) {
        QStringList::ConstIterator it=list.constBegin();
        for ( ; it!=list.constEnd(); ++it )
            if ( Global::myEmail(( *it ) ) )
                composer->setIdentity( Global::getIdentityForEmail( *it ) );
    }

    const QStringList cclist = msg->cc_list();
    if ( !cclist.isEmpty() ) {
        QStringList::ConstIterator it=cclist.begin();
        for ( ; it!=cclist.end(); ++it )
            if ( Global::myEmail(( *it ) ) )
                composer->setIdentity( Global::getIdentityForEmail( *it ) );
    }

    // set the attachments...
    QHash<KUrl,QString> list2 = msg->attachments();
    QHash<KUrl,QString>::ConstIterator ita;
    for ( ita = list2.constBegin(); ita != list2.constEnd(); ++ita )
        composer->addAttachment( ita.key(), ita.value() );
    composer->show();
}

void MainWindow::slotForwardMessageMenu()
{
    messageMenu( m_msgForwardActionMenu );
}

void MainWindow::slotReEditMessage( const QString& to )
{
    MessageWidget *i = m_tabWidget->currentTabWidget();

    MessageData *msg = i->msg();
    Composer *composer = new Composer( this, m_currentCollection );
    composer->setSubject( msg->subject() );
    composer->inReplyTo( msg->inreplyto() );

    KPIMIdentities::Identity id = Global::getIdentityForEmail( msg->sender_email() );
    if ( id != KPIMIdentities::Identity::null() ) {
        composer->setIdentity( id );
    }

    QString t;
    if ( !i->body()->page()->selectedText().isEmpty() )
        t = i->body()->page()->selectedText();
    else {
        t = msg->raw();
    }
    composer->setMsg( t, false );

    if ( !to.isEmpty() )
        composer->setRcpt( to, RecipientWidget::To );
    else
        composer->setRcpt( msg->to(), RecipientWidget::To );

    QHash<KUrl,QString> list2 = msg->attachments();
    QHash<KUrl,QString>::ConstIterator ita;
    for ( ita = list2.constBegin(); ita != list2.constEnd(); ++ita )
        composer->addAttachment( ita.key(), ita.value() );

    /*    Item item( msg->item() );
        item.setFlag( "\\Deleted" );
        Akonadi::ItemModifyJob *job = new Akonadi::ItemModifyJob( msg->item() );
        connect( job, SIGNAL( result( KJob* ) ), SLOT( slotResultStoreJob( KJob* ) ) );
        */

    composer->setDirty();
    composer->show();
}

void MainWindow::slotPurge()
{
    // use dbus to call the resource asking to slotPurge() the collection.
    kDebug() << "CollectionResource " << m_currentCollection.resource()
    << m_currentCollection.id();

    {
        QString dbusstr = "org.freedesktop.Akonadi.Resource.";
        dbusstr.append( m_currentCollection.resource() );
        QDBusInterface dbus( dbusstr, "/" );
        dbus.call( QLatin1String( "requestManualExpunge" ),
                   m_currentCollection.id() );
    }

    // Now the messages are removed from the imap server, but not yet in
    // the view.
    {
        QString dbusstr = "org.freedesktop.Akonadi.Resource.";
        dbusstr.append( m_currentCollection.resource() );
        QDBusInterface dbus( dbusstr, "/" );
        dbus.call( QLatin1String( "rsynchronizeCollection" ),
                   m_currentCollection.id() );
    }
}


void MainWindow::slotResultStoreJob( KJob *job )
{
    if ( job->error() ) {
        kWarning() << "Failed: " << job->errorString();
    }
}

void MainWindow::messageMenu( KActionMenu *am )
{
    am->menu()->clear();

    // add them from the current message.
    MessageWidget *i = m_tabWidget->currentTabWidget();
    QStringList values;
    bool sep = false;
    if ( i && i->msg() && i->headers()->isVisible() ) {
        MessageData *msg = i->msg();

        if ( !msg->replyTo().isEmpty() ) {
            values.append( msg->replyTo() );
            values.append( msg->replyTo() );
            sep = true;
        }

        if ( !msg->sender().isEmpty() ) {
            values.append( msg->sender() );
            values.append( msg->sender_email() );
            sep = true;
        }

        QStringList list = msg->to_list();
        if ( !list.isEmpty() ) {
            QStringList::ConstIterator it=list.constBegin();
            for ( ; it!=list.constEnd(); ++it ) {
                values.append( *it );
                values.append( *it );
                sep = true;
            }
        }

        list = msg->cc_list();
        if ( !list.isEmpty() ) {
            QStringList::ConstIterator it=list.constBegin();
            for ( ; it!=list.constEnd(); ++it ) {
                values.append( *it );
                values.append( *it );
                sep = true;
            }
        }
    }

    // PORT TO AKONADI
    /*
    // add top 10
    QStringList values2;
    m_db->getTopTenRecentList( values2 );

    if ( sep && values.count() > 0 ) {
        values.append( "-" );
        values.append( "-" );
    }

    QStringList::iterator it = values2.begin();
    while ( it != values2.end() ) {
        QString name = ( *it );
        ++it;
        QString email = ( *it );
        ++it;

        if ( !values.contains( email ) ) {
            values.append( name );
            values.append( email );
        }
    }

    // make the menu
    it = values.begin();
    while ( it != values.end() ) {
        QString name = ( *it );
        ++it;
        QString email = ( *it );
        ++it;

        if ( name.isEmpty() )
            name = email;

        if ( name == "-" )
            am->menu()->addSeparator();
        else
            am->menu()->addAction( name );
    }

    // still nothing???
    if ( values.isEmpty() ) {
        // change after messagefreeze
        QAction *i = am->menu()->addTitle( i18n( "None" ) );
        i->setEnabled( false );
    }
    */
}

void MainWindow::slotForwardMessageMenuSelected( QAction *i )
{
    slotForwardMessage( i->text() );
}

void MainWindow::slotNotifications()
{
    KNotifyConfigWidget::configure( this );
}

void MainWindow::slotSave()
{
    MessageWidget *tb = m_tabWidget->currentTabWidget();
    // kDebug() << ":save" << tb->msg()->subject();
    QString filename = KFileDialog::getSaveFileName( tb->msg()->subject(),
                       "*.eml|Email File\n*.txt|Text File", this, i18n( "Save as..." ) );
    tb->save( filename );
}

void MainWindow::slotPrint()
{
    m_tabWidget->currentTabWidget()->print();
}

#ifdef WITHNEPOMUMPLAYGROUND
void MainWindow::slotShowPossibleTags()
{
    MessageWidget *i =  m_tabWidget->currentTabWidget();
    NepomukWidget::showPossibleTagDialog( i->msg() );
    //i->updateNepomukWidget();
}
#endif

void MainWindow::slotCheckMail()
{
    kDebug() << "Implement me!";
}

void MainWindow::setActionStates()
{
    // Determine if a message is shown...
    bool message = m_tabWidget->isCurrentTab( TabWidget::Message );
    MessageWidget *tb = m_tabWidget->currentTabWidget();

    // Enable the actions.
    m_externalImageAction->setEnabled( message );
    m_plainTextAction->setEnabled( message );
    m_fixedfontAction->setEnabled( message );
    m_msgDelAction->setEnabled( message );
    m_msgSaveAction->setEnabled( message );
#ifdef WITHNEPOMUMPLAYGROUND
    m_msgPossibleTagAction->setEnabled( false /* it will get separate signals */ );
#endif
    m_msgPrintAction->setEnabled( message );
    m_msgZoomInAction->setEnabled( message );
    m_msgZoomOutAction->setEnabled( message );
    m_msgForwardAction->setEnabled( message );
    m_msgForwardActionMenu->setEnabled( message );
    m_msgReplyAction->setEnabled( message );
    m_msgReEditAction->setEnabled( message );
    m_sourceAction->setEnabled( message );
    m_msgUser0Action->setEnabled( message );
    m_msgUser1Action->setEnabled( message );
    m_msgUser2Action->setEnabled( message );
    m_msgUser3Action->setEnabled( message );
    m_msgUser4Action->setEnabled( message );
    m_msgUser5Action->setEnabled( message );

    // Set the state of the toggle icons.
    if ( message ) {
        m_plainTextAction->setEnabled( !m_sourceAction->isChecked() );
        m_fixedfontAction->setEnabled( m_plainTextAction->isChecked() );
        m_externalImageAction->setEnabled( !m_plainTextAction->isChecked() );
        m_msgDelAction->setChecked( m_itemList.value( tb ).hasFlag( "\\Deleted" ) );
        setStatusBarStates();
    }

    // And now for the mailbox actions.
    bool somethingSelected = m_collectionView->selectionModel()->hasSelection();
    m_syncResource->setEnabled( somethingSelected );
    m_purgeAction->setEnabled( somethingSelected );
    m_selectAllAction->setEnabled( somethingSelected );
    //m_childSortAction->setEnabled( somethingSelected );
    m_showDeletedAction->setEnabled( somethingSelected );
}

void MainWindow::slotEnablePossibleTagsAction( bool state )
{
#ifdef WITHNEPOMUMPLAYGROUND
    m_msgPossibleTagAction->setEnabled( state );
#else
    Q_UNUSED( state );
#endif
}

void MainWindow::slotAgentInstanceStatusChanged( const Akonadi::AgentInstance &instance )
{
    // kDebug() << instance.identifier() << instance.status() << instance.statusMessage();
    slotSimpleMessageStatusBar( i18nc( "Status message: first is the account, second the message shown",
                                       "[%1] %2", instance.name(), instance.statusMessage() ) );
}

void MainWindow::slotAgentInstanceProgressChanged( const Akonadi::AgentInstance &instance )
{
    Q_UNUSED( instance );
    // kDebug() << instance.identifier() << instance.progress() << instance.statusMessage();
}

void MainWindow::setStatusBarStates()
{
    if ( m_plainTextAction->isChecked() )
        m_statusBar->changeItem( i18nc( "Short for text-mode", "Plain" ), 3 );
    else
        m_statusBar->changeItem( i18nc( "Short for html-mode", "HTML" ), 3 );


    if ( m_externalImageAction->isChecked() )
        m_statusBar->changeItem( i18nc( "Short for external images", "Ext." ), 4 );
    else
        m_statusBar->changeItem( i18nc( "Short for no ext. images", "No ext." ), 4 );
}

void MainWindow::slotUpdateStatusBar( const QString &text )
{
    //kDebug() << "status: " << text;
    m_statusTimer->stop();
    m_lastMessage = text;

    if ( m_standAlone )
        m_statusText->setText( text );
}

void MainWindow::slotUpdateStatusBarRestoreMouse( const QString &text )
{
    //kDebug() << "status: " << text;
    if ( m_standAlone )
        m_statusText->setText( text );
    QApplication::restoreOverrideCursor();
    kapp->processEvents();
    m_statusTimer->start( 3000 );
}

void MainWindow::slotClearStatusBar()
{
    if ( m_standAlone )
        m_statusText->setText(
            i18nc( "task done","%1 [completed]", m_lastMessage ) );
    QApplication::restoreOverrideCursor();
    m_statusTimer->start( 1000 );
}

void MainWindow::slotSimpleMessageStatusBar( const QString &text )
{
    if ( m_standAlone )
        m_statusText->setText( text );
}

void MainWindow::slotStatusBarReady()
{
    if ( m_standAlone )
        m_statusText->setText( i18n( "Ready" ) );
}

void MainWindow::slotStatusBarClicked( int i )
{
    if ( i == 2 ) {
        slotQuickReply();
    } else if ( i == 3 ) {
        m_plainTextAction->setChecked( !m_plainTextAction->isChecked() );
        slotToggleHTML();
    } else if ( i == 4 ) {
        m_externalImageAction->setChecked( !m_externalImageAction->isChecked() );
        slotToggleExternalImage();
    }
}

void MainWindow::slotScrollBottom()
{
    m_headerWidget->view()->setUpdatesEnabled( false );
    m_headerWidget->view()->scrollToBottom();
    m_headerWidget->view()->setUpdatesEnabled( true );
}

void MainWindow::slotOpenURLInNewTab( const QString &page )
{
    KUrl kurl( page );
    QString title = kurl.hasHost() ? kurl.host() : i18n( "Site" );
    slotNewTab();

    MessageWidget *tb = m_tabWidget->currentTabWidget();
    slotUpdateTabTitle( title );
    tb->headers()->hide();
    tb->body()->setExternalImage( true );
    m_externalImageAction->setChecked( true );
    tb->body()->setOpenLinksDirectly( true );
    m_tabWidget->setCurrentTabType( TabWidget::Browser );
    if ( !page.isEmpty() )
        tb->body()->setUrl( page );
    setActionStates();
}

void MainWindow::slotUpdateTabTitle( const QString &title )
{
    if ( title.isEmpty() )
        return;
    m_tabWidget->setTabText( m_tabWidget->currentIndex(), title );
}

void MainWindow::collectionClicked( const Akonadi::Collection &collection )
{
    // kDebug() << collection.remoteId();
    if ( !collection.isValid() )
        return;
    m_currentCollection = collection;
    m_headerWidget->setCollection( collection );
    setActionStates();
}

void MainWindow::slotQuit()
{
    exit( 0 );
}

#include "mainwindow.moc"
