/*
   Copyright (C) 2006-2008 Omat Holding B.V. <info@omat.nl>
   Copyright (C) 2007 Frode M. Døving <frode@lnix.net>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*/

// Own
#include "messagebodywidget.h"
#include "contacttreemodel.h"
#include "contactcreation.h"
#include "modelmenu.h"

// Qt
#include <QMenu>
#include <QMouseEvent>
#include <QWebHitTestResult>

// KDE
#include <KConfigGroup>
#include <KDebug>
#include <KGlobalSettings>
#include <KMessageBox>
#include <KToolInvocation>
#include <kio/accessmanager.h>

#include <kpimutils/email.h>

using namespace Mailody;

MessageBodyWidget::MessageBodyWidget( MessageWidget* parent )
        : QWebView( parent ),
        m_currentMessage( 0 ),
        m_bodyType( MessageData::Plain ),
        m_fixedFont( false ),
        m_externalImage( false ),
        m_openDirectly( false )
{
    m_accessManager = new KIO::AccessManager( this );
    page()->setNetworkAccessManager( m_accessManager );
    page()->setLinkDelegationPolicy( QWebPage::DelegateAllLinks );

    KConfigGroup config = KGlobal::config()->group( "Messageview" );
    qreal factor = config.readEntry( "fontsize", 100 );
    setTextSizeMultiplier( factor/100 );

    show();

    connect( this, SIGNAL( titleChanged( const QString& ) ),
             SLOT( slotSetCaption( const QString& ) ) );
    connect( page(), SIGNAL( linkHovered( const QString&, const QString&, const QString& ) ),
             SLOT( slotLink( const QString&, const QString&, const QString& ) ) );

    m_addressbookModel = parent->addressbookModel();
}

MessageBodyWidget::~MessageBodyWidget()
{
    KConfigGroup config = KGlobal::config()->group( "Messageview" );
    config.writeEntry( "fontsize", textSizeMultiplier()*100 );
    config.sync();
}

void MessageBodyWidget::slotSetCaption( const QString& title )
{
    emit pageTitle( title );
}

void MessageBodyWidget::setMsg( MessageData* msg )
{
    m_currentMessage = msg;
    const QString data = m_currentMessage->requestBody( m_bodyType );
    m_openDirectly = false;

    setHtml( data.isEmpty() ? "&nbsp;" : data );
}

void MessageBodyWidget::showHTML( bool i )
{
    i ? m_bodyType = MessageData::HTML : m_bodyType = MessageData::Plain;
    if ( m_currentMessage )
        setHtml( m_currentMessage->requestBody( m_bodyType ) );
}

void MessageBodyWidget::setViewSource()
{
    m_bodyType = MessageData::Source;

    if ( m_currentMessage )
        setHtml( m_currentMessage->requestBody( m_bodyType ) );
}

void MessageBodyWidget::setFixedFont( bool i )
{
    m_fixedFont = i;
    settings()->setFontFamily( QWebSettings::StandardFont,
                               i ? KGlobalSettings::fixedFont().family() : KGlobalSettings::generalFont().family() );

    if ( m_currentMessage ) {
        setHtml( m_currentMessage->body() );
    }
}

void MessageBodyWidget::setExternalImage( bool i, bool screenupdate )
{
    m_externalImage = i;
    m_accessManager->setExternalContentAllowed( i );

    if ( m_currentMessage && screenupdate ) {
        setHtml( m_currentMessage->body() );
    }
}

void MessageBodyWidget::setOpenLinksDirectly( bool i )
{
    m_openDirectly = i;
}

void MessageBodyWidget::slotLink( const QString & link, const QString &, const QString & textContent )
{
    QString statusText =  textContent.isEmpty() ? link : textContent;
    statusText.remove( '\n' );
    emit status( statusText );
}


void MessageBodyWidget::mousePressEvent( QMouseEvent * event )
{
    // stuff can be handled by the contextMenu event.
    QWebHitTestResult webHit = page()->mainFrame()->hitTestContent( event->pos() );
    QString url = webHit.linkUrl().toString();
    if ( url.isEmpty() || event->button() == Qt::RightButton ) {
        event->ignore();
        QWebView::mousePressEvent( event );
        return;
    }
    event->accept();

    QString linkText = webHit.linkText();

    // if the text of the link contains http:// or www., match that against the
    // actual url. If that does not contain it, we might have a case of
    // phishing. I know it's not waterproof, but it's a start.
    if (( linkText.indexOf( "http" ) > -1 || linkText.indexOf( "www" ) > -1 ) &&
            linkText.indexOf( url ) == -1 ) {
        kDebug() << "PHISHING alert" << url << linkText;
        int i = KMessageBox::warningYesNo( 0,
                                           i18n( "You have clicked on a link which might not indicate "
                                                 "correctly where you are really going to. Please check "
                                                 "if you really want to view a page on this server:\n\n"
                                                 "%1\n\nDo you want to go there?", url ),
                                           i18n( "Phishing Alert" ) );
        if ( i == KMessageBox::No )
            return;
    }

    // if the text of the link is an email address, the link should also contain that email
    // address. If not, this is also a kind of phishing in my eyes.
    if ( KPIMUtils::isValidSimpleAddress( linkText ) && linkText.indexOf( url ) == -1 ) {
        kDebug() << "PHISHING alert" << url << linkText;
        int i = KMessageBox::warningYesNo( 0,
                                           i18n( "You have clicked on a link which looks like an email "
                                                 "address, but the link behind it does not match this:\n\n"
                                                 "%1\n\nDo you want to follow the original link or simply compose a "
                                                 "new email to %2?", url, linkText ),
                                           i18n( "Phishing Alert" ), KGuiItem( i18n( "Compose Email" ) ),
                                           KGuiItem( i18n( "Follow original link" ) ) );
        if ( i == KMessageBox::Yes )
            url = "mailto:" + linkText;
    }

    if ( url.startsWith( QLatin1String( "mailto:" ) ) ) {
        emit openComposer( url.mid( 7 ) );
    } else {
        if ( event->button() == Qt::MidButton )
            emit openInTab( url );
        else if ( m_openDirectly )
            setUrl( webHit.linkUrl() );
        else
            KToolInvocation::invokeBrowser( url );
    }
}

void MessageBodyWidget::contextMenuEvent( QContextMenuEvent * event )
{

    QMenu* p = new QMenu( 0 );

    QAction* newmsg = new QAction( i18n( "New Message to..." ), this );
    newmsg->setIcon( KIcon( "mail-message-new" ) );
    QAction* newurl = new QAction( i18n( "Open" ), this );
    QAction* newtab = new QAction( i18n( "Open in New Tab" ), this );
    QAction* zoomin = new QAction( i18n( "Zoom In" ), this );
    zoomin->setIcon( KIcon( "zoom-in" ) );
    QAction* zoomout = new QAction( i18n( "Zoom Out" ), this );
    zoomout->setIcon( KIcon( "zoom-out" ) );

    ModelMenu* addressbookMenu = new ModelMenu( this );
    addressbookMenu->setModel( m_addressbookModel );

    QWebHitTestResult webHit = page()->mainFrame()->hitTestContent( event->pos() );
    QString url = webHit.linkUrl().toString();
    if ( url.startsWith( QLatin1String( "mailto:" ) ) ) {
        p->addAction( newmsg );

        QAction* add = p->addMenu( addressbookMenu );
        add->setText( i18n( "Add to address book" ) );
        add->setIcon( KIcon( "contact-new" ) );
    } else if ( !url.isEmpty() ) {
        p->addAction( newurl );
        p->addAction( newtab );
        p->addAction( page()->action( QWebPage::CopyLinkToClipboard ) );
    }

    p->addSeparator();

    if ( !webHit.imageUrl().isEmpty() ) {
        p->addAction( page()->action( QWebPage::CopyImageToClipboard ) );
        p->addSeparator();
    }

    if ( !selectedText().isEmpty() ) {
        p->addAction( page()->action( QWebPage::Copy ) );
        p->addSeparator();
    }

    p->addAction( zoomin );
    p->addAction( zoomout );

    QAction* choice = p->exec( mapToGlobal( event->pos() ) );

    QModelIndex index = addressbookMenu->index( choice );
    if ( index.isValid() ) {
        const Akonadi::Collection collection =
            index.data( Akonadi::EntityTreeModel::CollectionRole ).value<Akonadi::Collection>();
        kDebug() << "Adding to Collection" << collection.name();
        new ContactCreation( this, url.mid( 7 ), collection );
    } else if ( choice == newmsg ) {
        emit openComposer( url.mid( 7 ) );
    } else if ( choice == newurl ) {
        KToolInvocation::invokeBrowser( url );
    } else if ( choice == newtab )
        emit openInTab( url );
    else if ( choice == zoomin )
        action( ZoomIn );
    else if ( choice == zoomout )
        action( ZoomOut );
    delete p;
}

void MessageBodyWidget::clearView()
{
    setHtml( "&nbsp;" );
}

void MessageBodyWidget::action( KeyAction action )
{
    int current = page()->currentFrame()->scrollBarValue( Qt::Vertical );
    int height = page()->viewportSize().height();
    switch ( action ) {
    case ScrollUp:
        page()->currentFrame()->setScrollBarValue( Qt::Vertical, current - 10 );
        break;
    case ScrollDown:
        page()->currentFrame()->setScrollBarValue( Qt::Vertical, current + 10 );
        break;
    case ScrollPageUp:
        page()->currentFrame()->setScrollBarValue( Qt::Vertical, current+height*-0.85 );
        break;
    case ScrollPageDown:
        page()->currentFrame()->setScrollBarValue( Qt::Vertical, current+height*0.85 );
        break;
    case ZoomIn:
        setTextSizeMultiplier( textSizeMultiplier() + .1 );
        break;
    case ZoomOut:
        setTextSizeMultiplier( textSizeMultiplier() - .1 );
        break;
    }
}

#include "messagebodywidget.moc"
