/*
   Copyright (C) 2006-2008 Omat Holding B.V. <info@omat.nl>
   Copyright (C) 2007 Frode M. Døving <frode@lnix.net>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*/

#ifndef MESSAGEBODYWIDGET_H
#define MESSAGEBODYWIDGET_H

#include "messagedata.h"
#include "messagewidget.h"

#include <QWebView>
#include <kio/accessmanager.h>

namespace Mailody
{

class ContactTreeModel;

/**
 * @class MessageBodyWidget
 * This class is responsible for displaying the body of the message
 * to achieve this, it inherits a KHTMLPart
 * part of the TotalView class.
 */
class MessageBodyWidget : public QWebView
{
    Q_OBJECT

public:
    enum KeyAction { ScrollUp=0, ScrollDown,
                     ScrollPageUp, ScrollPageDown,
                     ZoomIn, ZoomOut
                   };

    /**
    * Constructor
    */
    explicit MessageBodyWidget( MessageWidget* );

    /**
    * Destructor
    */
    ~MessageBodyWidget();

    /**
     * Scrolls to the top of the message
     */
    void top()             { /*TODO: setContentsPos(0,0); */};

    /**
     * Returns if the message html will be interepreted or displayed as
     * plain text (escaped).
     */
    bool html() const      {
        return m_bodyType == MessageData::HTML;
    };

    /**
     * Returns if the source of the message will be showed
     */
    bool source() const    {
        return m_bodyType == MessageData::Source;
    };

    /**
     * Returns if the message will be shown in fixed font, has no use
     * in html mode of course
     */
    bool fixedfont() const {
        return m_fixedFont;
    };

    /**
     * Returns if the message will load external images in html-mode
     */
    bool externalImage() const {
        return m_externalImage;
    };

    /**
     * Sets the message to be shown, remember that you probably want to
     * use TotalView::setMsg() to also get updated headers
     * @param msg the message to show
     */
    void setMsg( MessageData* msg );

    /**
     * Indicate how the message is rendered.
     * @param i If true the message is rendered in html
     */
    void showHTML( bool i );

    /**
     * set @p i to true to load external images in the message
     * it will update the message immediately, unless @p updateScreen
     * is set to false.
     * @param i Set to true to load external images.
     * @param updateScreen Possibility to not directly repaint the screen if more actions are wanted.
     */
    void setExternalImage( bool i, bool updateScreen=true );

    /**
     * set this so the links will open directly, normaly it would
     * open a link in a external browser or emit the signal on middle
     * click
     * @param i If true links are opened directly.
     */
    void setOpenLinksDirectly( bool i );

    /**
     * set @p i to true to see the message in a fixed font
     * @param i If true the message will be shown in fixed font.
     */
    void setFixedFont( bool i );

    /**
     * Call this to see the source of the message
     */
    void setViewSource();

    /**
     * this will clear the body.
     */
    void clearView();

    /**
     * Functions
     */
    void action( KeyAction );

protected:
    void contextMenuEvent( QContextMenuEvent * event );
    void mousePressEvent( QMouseEvent * event );


// @cond private
private slots:
    void slotSetCaption( const QString& title );
    void slotLink( const QString & link, const QString & title, const QString & textContent );
// @endcond

signals:
    /**
     * Emitted when clicked on an address
     * @param email the emailaddress clicked on.
     */
    void openComposer( const QString &email );

    /** Emitted when middle clicked on an url in the message. **/
    void openInTab( const QString& );

    /** Emitted to indicate the tab title **/
    void pageTitle( const QString& );

    /** Emits the status **/
    void status( const QString& );

private:
    // @cond private
    MessageData*             m_currentMessage;
    ContactTreeModel        *m_addressbookModel;
    MessageData::bodyType    m_bodyType;
    bool                     m_fixedFont;
    bool                     m_externalImage;
    bool                     m_openDirectly;
    KIO::AccessManager*      m_accessManager;
    // @endcond
};

}

#endif
