/*
   Copyright (C) 2006-2009 Omat Holding B.V. <info@omat.nl>
   Copyright (C) 2007 Frode M. Døving <frode@lnix.net>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*/

// Own
#include "messageheaderwidget.h"
#include "contacttreemodel.h"
#include "contactcreation.h"
#include "modelmenu.h"

// Qt
#include <QApplication>
#include <QClipboard>
#include <QLabel>
#include <QMenu>
#include <QScrollBar>
#include <QTextBrowser>
#include <QTextDocument> // Qt::escape()

// KDE
#include <KDebug>
#include <KConfigGroup>
#include <KFileDialog>
#include <KRun>

#include <kpimutils/linklocator.h>
#include <kio/netaccess.h>

using namespace Mailody;

MessageHeaderWidget::MessageHeaderWidget( MessageWidget* parent )
        : QTextBrowser( parent ), m_currentMessage( 0 ),  m_storeAttachments( false )
{
    hide();
    setHorizontalScrollBarPolicy( Qt::ScrollBarAlwaysOff );
    setLineWidth( 1 );
    setFrameStyle( QFrame::Panel | QFrame::Sunken );

    m_foto = new QLabel( this );

    setOpenLinks( false );
    connect( this, SIGNAL( anchorClicked( const QUrl & ) ),
             SLOT( slotLeftMouseClick( const QUrl& ) ) );

    setContextMenuPolicy( Qt::CustomContextMenu );
    connect( this, SIGNAL( customContextMenuRequested( const QPoint & ) ),
             SLOT( slotRightMouseClick( const QPoint& ) ) );

    m_addressbookModel = parent->addressbookModel();
}

MessageHeaderWidget::~MessageHeaderWidget()
{}

void MessageHeaderWidget::setMsg( MessageData* msg )
{
    m_currentMessage = msg;
    verticalScrollBar()->setValue( 0 );
    horizontalScrollBar()->setValue( 0 );

    if ( msg && isHidden() ) {
        show(); // can be hidden by favorite website / middle click.
    }

    // keep it after the possible show(), so the first message is
    // shown correctly too.
    updateHeaders();
}

void MessageHeaderWidget::autoSaveAttachments()
{
    KConfigGroup config = KGlobal::config()->group( "General" );
    m_storeAttachments = config.readEntry( "storeAllAttachments", false );
    m_attachmentFolder = KUrl::fromPath( config.readEntry( "attachmentsFolder" ) );
}

void MessageHeaderWidget::getFreeFilename( KUrl &fileLocation )
{
    // now we see if a file with that name already exists.
    const QString path = fileLocation.path();
    bool exists = KIO::NetAccess::exists( fileLocation, KIO::NetAccess::DestinationSide, this );
    int i = 0;
    while ( exists ) {
        // it exists, so find the extension, add a digit and try again.
        int extAt = path.lastIndexOf( '.' );
        if ( extAt == -1 )
            extAt = path.length();

        const QString newFilename = path.left( extAt ) + '_' + QString::number( i ) + path.mid( extAt );
        fileLocation.setPath( newFilename );

        exists = KIO::NetAccess::exists( fileLocation, KIO::NetAccess::DestinationSide, this );
        i++;
    }
}

void MessageHeaderWidget::autoSaveAttachment( const KUrl &fileLocation, const QString &fileName )
{
    KUrl destination( m_attachmentFolder );
    destination.addPath( fileName );
    getFreeFilename( destination );

    kDebug() << "Filelocation" << fileLocation << "Visible Name" << fileName << "Destination" << destination;

    KIO::NetAccess::file_copy( fileLocation, destination, this );
}

void MessageHeaderWidget::updateHeaders()
{
    if ( !m_currentMessage )
        return;

    //------------- the rest
    using KPIMUtils::LinkLocator;
    const int flags = LinkLocator::PreserveSpaces |
                      LinkLocator::HighlightText |
                      LinkLocator::IgnoreUrls;

    QString text = "<table style=\"margin-right: 80px; margin-left:4px;\" ";
    text.append( "cellpadding=0 cellspacing=1 width=\""
                 + QString::number( width() ) + "\">" );
    text.append( "<tr><td colspan=2>"
                 + LinkLocator::convertToHtml( m_currentMessage->subject()
                                               .trimmed(), flags )
                 + "</td></tr>" );

    QString t = "<a href=\"mailto:"
                + LinkLocator::convertToHtml(
                    m_currentMessage->sender_full( false ),
                    flags ) + "\">"
                + LinkLocator::convertToHtml(
                    m_currentMessage->sender_full( true )
                    .trimmed(), flags ) + "</a>";
    text.append( "<tr><td valign=top align=right>" + i18n( "From" ) + ": </td>" );
    if ( m_currentMessage->userAgent().startsWith( QLatin1String( "Mailody" ) ) )
        text.append( "<td valign=top width=\"100%\"><font color=\"#22cc22\">"
                     + t + "<td></tr>\n" );
    else
        text.append( "<td valign=top width=\"100%\">" + t + "</td></tr>" );

    const QStringList to = m_currentMessage->to_list();
    QString toList;
    if ( to.count()>0 ) {
        QStringList::ConstIterator it=to.constBegin();
        for ( ; it!=to.constEnd(); ++it ) {
            if ( !toList.isEmpty() )
                toList.append( ", " );
            toList.append( "<a href=\"mailto:"
                           + LinkLocator::convertToHtml(( *it ).trimmed(), flags )
                           + "\">"
                           + LinkLocator::convertToHtml(( *it ).trimmed(), flags )
                           + "</a>" );
        }
        text.append( "<tr><td align=right valign=top>" + i18n( "To" ) + ": </td>" );
        text.append( "<td valign=top width=\"100%\">" + toList + "</td></tr>" );
    }

    const QStringList cc = m_currentMessage->cc_list();
    if ( cc.count()>0 ) {
        QString t;
        QStringList::ConstIterator it=cc.constBegin();
        for ( ; it!=cc.constEnd(); ++it ) {
            if ( !t.isEmpty() )
                t.append( ", " );
            t.append( "<a href=\"mailto:"
                      + LinkLocator::convertToHtml(( *it ).trimmed(), flags )
                      + "\">"
                      + LinkLocator::convertToHtml(( *it ).trimmed(), flags )
                      + "</a>" );

        }
        text.append( "<tr><td valign=top align=right>"
                     + i18n( "Cc" )+": </td>" );
        text.append( "<td valign=top width=\"100%\">" + t + "</td></tr>" );
    }

    const QString resentFrom = m_currentMessage->resentFrom().trimmed();
    if ( !resentFrom.isEmpty() ) {
        const QString t = "<a href=\"mailto:"
                          + LinkLocator::convertToHtml( resentFrom, flags ) +"\">"
                          + LinkLocator::convertToHtml( resentFrom, flags ) + "</a>";

        QString t2;
        const QString resentTo = m_currentMessage->resentTo().trimmed();
        if ( resentTo.isEmpty() )
            t2 = toList;
        else
            t2 = "<a href=\"mailto:"
                 + LinkLocator::convertToHtml( resentTo, flags ) +"\">"
                 + LinkLocator::convertToHtml( resentTo, flags ) + "</a>";

        text.append( "<tr><td valign=top align=right><nobr>" +
                     i18n( "Resent" )+": </nobr></td>" );
        text.append( "<td valign=top>"
                     + i18nc( "resent by from-address(1) to to-address(2)",
                              "by %1 to %2", t, t2 ) + "</td></tr>" );
    }

    const QHash<KUrl,QString> list = m_currentMessage->attachments();
    QHash<KUrl,QString>::ConstIterator ita;
    if ( !list.isEmpty() ) {
        QString t;
        for ( ita = list.constBegin(); ita != list.constEnd(); ++ita ) {
            if ( !t.isEmpty() )
                t.append( ", " );
            t.append( "<a href=\"attachment:"+ita.key().path()+"\">" +
                      Qt::escape( ita.value().trimmed() ) + "</a>" );

            if ( m_storeAttachments )
                autoSaveAttachment( ita.key(), ita.value() );
        }
        text.append( "<tr><td align=right valign=top>"
                     + i18n( "Attachments" )+": </td>" );
        text.append( "<td valign=top align=left>" + t + "</td></tr>" );
    }

    text.append( "</table>" );

    setText( text );
}

void MessageHeaderWidget::clearView()
{
    setText( "<table width=\"100%\" height=\"100%\">"
             "<tr height=\"100%\"><td width=\"100%\">&nbsp;</td></tr></table>" );
    m_foto->setText( QString() );
    hide();
}

void MessageHeaderWidget::slotLeftMouseClick( const QUrl& to )
{
    kDebug() << to << to.path() << to.scheme();
    if ( !m_currentMessage )
        return;

    if ( to.scheme().compare( "mailto", Qt::CaseSensitive ) == 0 ) {
        emit openComposer( to.path() );
    } else if ( to.scheme().compare( "attachment",Qt::CaseSensitive ) == 0 ) {
        KRun *run = new KRun( to.path(), this );
        run->setRunExecutables( false );
    }
}

void MessageHeaderWidget::slotRightMouseClick( const QPoint& pos )
{
    const QString to = anchorAt( pos );
    kDebug() << to << endl;
    if ( to.isEmpty() )
        return;

    if ( to.startsWith( QLatin1String( "mailto:" ) ) ) {
        QString address = to.mid( 7 );

        QMenu* p = new QMenu( this );

        QAction* newmsg = new QAction( i18n( "New Message to..." ), this );
        newmsg->setIcon( KIcon( "mail-message-new" ) );

        QAction* copy = new QAction( i18n( "Copy" ), this );
        copy->setIcon( KIcon( "edit-copy" ) );

        ModelMenu* addressbookMenu = new ModelMenu( this );
        addressbookMenu->setModel( m_addressbookModel );
        QAction* add = p->addMenu( addressbookMenu );
        add->setText( i18n( "Add to address book" ) );
        add->setIcon( KIcon( "contact-new" ) );

        p->addAction( newmsg );
        p->addAction( add );
        p->addSeparator();
        p->addAction( copy );

        QAction* choice = p->exec( QCursor::pos() );

        QModelIndex index = addressbookMenu->index( choice );
        if ( index.isValid() ) {
            const Akonadi::Collection collection =
                index.data( Akonadi::EntityTreeModel::CollectionRole ).value<Akonadi::Collection>();
            kDebug() << "Adding to Collection" << collection.name();
            new ContactCreation( this, address, collection );
        } else if ( choice == newmsg )
            slotLeftMouseClick( to );
        else if ( choice == copy ) {
            QClipboard *cb = QApplication::clipboard();
            cb->setText( address, QClipboard::Clipboard );
        }
    } else if ( to.startsWith( QLatin1String( "attachment:" ) ) ) {
        QString attachment = to.mid( 11 );

        QMenu* p = new QMenu( this );
        QAction* open = new QAction( i18n( "Open" ), this );
        QAction* openwith = new QAction( i18n( "Open with..." ), this );
        QAction* save = new QAction( i18n( "Save..." ), this );
        QAction* saveAll = new QAction( i18n( "Save All..." ), this );

        p->addAction( open );
        p->addAction( openwith );
        p->addAction( save );
        p->addAction( saveAll );

        QAction* choice = p->exec( QCursor::pos() );

        KUrl url( attachment );
        if ( choice == open ) {
            KRun *run = new KRun( url, this );
            run->setRunExecutables( false );
        } else if ( choice == openwith ) {
            KRun::displayOpenWithDialog( KUrl::List::List( url ), this );
        } else if ( choice == save ) {
            const QString filename = m_currentMessage->attachments().value( url );
            KMimeType::Ptr mimeType = KMimeType::findByUrl( url );
            KFileDialog dialog( KUrl( "kfiledialog:///saveAttachmentsFolder" ),
                                mimeType->name(), this );
            dialog.setSelection( filename );
            dialog.setOperationMode( KFileDialog::Saving );
            dialog.setMode( KFile::File );
            dialog.setConfirmOverwrite( KFileDialog::ConfirmOverwrite );
            dialog.setCaption( i18n( "Save As" ) );
            dialog.exec();
            const KUrl destination = dialog.selectedUrl();
            if ( destination.isValid() )
                KIO::NetAccess::file_copy( url, destination, this );
        } else if ( choice == saveAll ) {
            KUrl base = KFileDialog::getExistingDirectoryUrl(
                            KUrl( "kfiledialog:///saveAttachmentsFolder" ), this );
            if ( !base.hasPath() )
                return;

            const QHash<KUrl,QString> aMap = m_currentMessage->attachments();
            QHash<KUrl,QString>::ConstIterator ita;
            for ( ita = aMap.constBegin(); ita != aMap.constEnd(); ++ita ) {
                KUrl destination( base );
                destination.addPath( ita.value() );
                getFreeFilename( destination );

                kDebug() << "copy: " << ita.key() << "->" << destination;

                KIO::NetAccess::file_copy( ita.key(), destination, this );
            }
        }
    }
}

#include "messageheaderwidget.moc"
