/*
   Copyright (C) 2006-2009 Omat Holding B.V. <info@omat.nl>
   Copyright (C) 2007 Frode M. Døving <frode@lnix.net>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*/

// Own
#include "messagewidget.h"
#include "contacttreemodel.h"
#include "messagebodywidget.h"
#include "messageheaderwidget.h"

// Qt
#include <QPrintPreviewDialog>
#include <QPointer>
#include <QSplitter>
#include <QTextCodec>
#include <QWebFrame>

// KDE
#include <KDebug>
#include <KGlobalSettings>
#include <kio/accessmanager.h>

#include <kabc/addressee.h>
#include <akonadi/changerecorder.h>
#include <akonadi/itemfetchscope.h>
#include <akonadi/session.h>

using namespace Mailody;

MessageWidget::MessageWidget( QWidget* parent )
        : QWidget( parent ), m_msg( 0 )
{
    Akonadi::ChangeRecorder *monitor = new Akonadi::ChangeRecorder( this );
    monitor->setCollectionMonitored( Akonadi::Collection::root() );
    monitor->setMimeTypeMonitored( KABC::Addressee::mimeType(), true );

    m_addressbookModel = new ContactTreeModel( monitor, this );
    m_addressbookModel->setIncludeRootCollection( false );
    m_addressbookModel->setItemPopulationStrategy( ContactTreeModel::NoItemPopulation );

    // create the layout.
    m_splitter = new QSplitter( this );
    m_splitter->setOrientation( Qt::Vertical );
    m_headers = new MessageHeaderWidget( this );
    m_splitter->addWidget( m_headers );
    m_body = new MessageBodyWidget( this );
    m_splitter->addWidget( m_body );

    KConfigGroup configMain = KGlobal::config()->group( "MainWindowSizes" );
    QList<int> defaultsize = configMain.readEntry( "messagesplitter", QList<int>() );
    if ( defaultsize.isEmpty() ) {
        defaultsize.append( 150 );
        defaultsize.append( 600 );
    }
    m_splitter->setSizes( defaultsize );

    autoSaveAttachments();
}

MessageWidget::~MessageWidget()
{
    KConfigGroup configMain = KGlobal::config()->group( "MainWindowSizes" );
    configMain.writeEntry( "messagesplitter", m_splitter->sizes() );
}

void MessageWidget::setMsg( MessageData* msg )
{
    m_msg = msg;
    m_headers->setMsg( m_msg );
    m_body->setMsg( m_msg );
}

void MessageWidget::clearView()
{
    m_headers->clearView();
    m_body->clearView();
}

void MessageWidget::save( const QString& filename )
{
    QFile file( filename );
    QTextStream stream( &file );
    stream.setCodec( QTextCodec::codecForName( "UTF-8" ) );
    file.open( QIODevice::WriteOnly );
    stream << m_msg->raw().trimmed();
    file.close();
}

void MessageWidget::print()
{
    QString msgheadersdata;
    msgheadersdata += "<table border=1 cellspacing=0 "
                      "cellpadding=0 width=100%>";
    msgheadersdata += "<tr><td colspan=2 bgcolor=\"#dddddd\"><b><h2>"
                      + msg()->subject() + "<h2></b></td>";
    msgheadersdata += "</tr>";
    msgheadersdata += "<tr><td align=right><b>" + i18n( "From:" ) + "</b></td>";
    msgheadersdata += "    <td width=\"100%\">" + msg()->sender_full( true )
                      + "</td>";
    msgheadersdata += "</tr>";
    msgheadersdata += "<tr><td align=right><b>" + i18n( "To:" ) + "</b></td>";
    msgheadersdata += "    <td width=\"100%\">" + msg()->to() + "</td>";
    msgheadersdata += "</tr>";
    if ( !msg()->cc().isEmpty() ) {
        msgheadersdata += "<tr><td align=right><b>" + i18n( "Cc:" )
                          + "</b></td>";
        msgheadersdata += "    <td width=\"100%\">" + msg()->cc() + "</td>";
        msgheadersdata += "</tr>";
    }
    msgheadersdata += "<tr><td align=right><b>" + i18n( "Date:" ) + "</b></td>";
    msgheadersdata += "    <td width=\"100%\">" + msg()->vDate() + "</td>";
    msgheadersdata += "</tr></table>";

    m_printPart = new QWebPage( this );
    KIO::AccessManager* accessManager = new KIO::AccessManager( this );
    accessManager->setExternalContentAllowed( body()->externalImage() );
    m_printPart->setNetworkAccessManager( accessManager );
    m_printPart->settings()->setFontFamily( QWebSettings::StandardFont,
                                            m_body->fixedfont() ? KGlobalSettings::fixedFont().family() : KGlobalSettings::generalFont().family() );

    m_printPart->mainFrame()->setHtml( msgheadersdata  + "<br><br>" + msg()->body() );
    connect( m_printPart, SIGNAL( loadFinished( bool ) ), this, SLOT( slotPrintPreview() ) );
}

void MessageWidget::slotPrintPreview()
{
    QPointer<QPrintPreviewDialog> dlg = new QPrintPreviewDialog( m_printPart->view() );
    connect( dlg, SIGNAL( paintRequested( QPrinter * ) ),
             m_printPart->mainFrame(), SLOT( print( QPrinter * ) ) );
    dlg->exec();
    delete dlg;
    delete m_printPart;
}

void MessageWidget::autoSaveAttachments()
{
    m_headers->autoSaveAttachments();
}

#include "messagewidget.moc"
