/*
   Copyright (C) 2009 Omat Holding B.V. <info@omat.nl>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*/

#include "recentmanager.h"
#include "recentattributes.h"

#include <QDBusInterface>

#include <akonadi/attributefactory.h>
#include <akonadi/agentmanager.h>
#include <akonadi/agentinstancecreatejob.h>
#include <akonadi/collection.h>
#include <akonadi/collectionfetchjob.h>
#include <akonadi/collectionfetchscope.h>
#include <akonadi/itemcreatejob.h>
#include <akonadi/itemfetchscope.h>
#include <akonadi/itemfetchjob.h>
#include <akonadi/itemmodifyjob.h>
#include <kabc/addressee.h>
#include <KStandardDirs>

using namespace Mailody;
using namespace Akonadi;

//@cond private
RecentManager *RecentManager::m_instance = 0;
const QLatin1String recentName( "MailodyRecentAddressBook" );
//@endcond

RecentManager::RecentManager() :
        QObject( 0 ), m_collection( Akonadi::Collection() )
{
    AttributeFactory::registerAttribute<AmountAttribute>();
    AttributeFactory::registerAttribute<DateAttribute>();
}

RecentManager* RecentManager::instance()
{
    if ( !m_instance )
        m_instance = new RecentManager();
    return m_instance;
}

void RecentManager::getRecentCollection()
{
    if ( m_collection.isValid() ) {
        emit recentCollection( m_collection );
        return;
    }

    // Try to find a resource that has the name 'MailodyRecentAddressBook'
    AgentManager *manager = AgentManager::self();
    AgentInstance recentResource;
    foreach( const AgentInstance& agent, manager->instances() ) {
        if ( agent.name() == recentName )
            recentResource = agent;
    }

    if ( !recentResource.isValid() ) { // create the addressbook
        AgentType type = AgentManager::self()->type( "akonadi_vcard_resource" );
        AgentInstanceCreateJob *job = new AgentInstanceCreateJob( type );
        connect( job, SIGNAL( result( KJob * ) ),
                 SLOT( slotRecentResourceCreated( KJob * ) ) );
        job->start();
    } else {
        getCollectionFromRecentResource( recentResource.identifier() );
    }
}

void RecentManager::slotRecentResourceCreated( KJob *job )
{
    if ( !job->error() ) {
        AgentInstance instance =
            static_cast<AgentInstanceCreateJob*>( job )->instance();
        instance.setName( recentName );
        QDBusInterface dbus( "org.freedesktop.Akonadi.Agent." + instance.identifier(),
                             "/Settings" );
        QDBusMessage reply = dbus.call( QLatin1String( "setPath" ),
                                        KStandardDirs::locateLocal( "data", "mailody/recent.vcf" ) );
        if ( reply.type() == QDBusMessage::ErrorMessage ) {
            kDebug() << reply.errorMessage();
        } else {
            instance.synchronizeCollectionTree();
            getCollectionFromRecentResource( instance.identifier() );
        }
    }
}

void RecentManager::getCollectionFromRecentResource( const QString& identifier )
{
    if ( identifier.isEmpty() )
        return;

    CollectionFetchScope scope;
    scope.setResource( identifier );

    CollectionFetchJob *job =
        new CollectionFetchJob( Collection::root(),
                                CollectionFetchJob::FirstLevel, this );
    job->setFetchScope( scope );
    connect( job, SIGNAL( collectionsReceived( Akonadi::Collection::List ) ),
             SLOT( slotRecentCollectionsReceived( Akonadi::Collection::List ) ) );
    job->start();
}

void RecentManager::slotRecentCollectionsReceived( Collection::List list )
{
    m_collection = list.at( 0 );
    emit recentCollection( m_collection );
}

void RecentManager::add( const QString& email, const QString& name )
{
    if ( !m_collection.isValid() )
        return;

    Akonadi::ItemFetchJob *job = new Akonadi::ItemFetchJob( m_collection );
    job->fetchScope().fetchFullPayload();
    job->fetchScope().fetchAllAttributes( true );

    if ( job->exec() ) {
        bool found = false;

        DateAttribute* date = new DateAttribute;
        date->setDate( QDateTime::currentDateTime() );

        Akonadi::Item::List items = job->items();
        foreach( const Akonadi::Item &existingItem, items ) {

            if ( !existingItem.hasPayload<KABC::Addressee>() )
                continue;

            KABC::Addressee addressee = existingItem.payload<KABC::Addressee>();
            if ( addressee.preferredEmail().toLower() != email.toLower() )
                continue;

            found = true;

            Akonadi::Item item = existingItem;
            if ( existingItem.hasAttribute<AmountAttribute>() ) {

                AmountAttribute* amount = new AmountAttribute;
                int a = existingItem.attribute<AmountAttribute>()->amount();
                a++;
                amount->setAmount( a );
                item.addAttribute( amount );
                item.addAttribute( date );
            } else {

                AmountAttribute* amount = new AmountAttribute;
                amount->setAmount( 1 );
                item.addAttribute( amount );
                item.addAttribute( date );
            }

            new Akonadi::ItemModifyJob( item );
            break;

        }

        if ( !found ) {
            KABC::Addressee addressee;
            addressee.setName( name );
            addressee.setEmails( QStringList( email.toLower() ) );

            AmountAttribute* amount = new AmountAttribute;
            amount->setAmount( 1 );

            Item item( -1 );
            item.setMimeType( "text/directory" );
            item.setPayload<KABC::Addressee>( addressee );
            item.addAttribute( amount );
            item.addAttribute( date );

            new ItemCreateJob( item, m_collection );
        }

    } else {
        kDebug() << "Error occurred";
    }


}

#include "recentmanager.moc"
