/* This file is part of the KDE project

   Copyright (C) 2006-2009 Omat Holding B.V. <info@omat.nl>
   Copyright (C) 2007 Frode M. Døving <frode@lnix.net>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*/

#include "recipientwidget.h"

#include "addresslineedit.h"
#include "composer.h"
#include "composerrecipientwidget.h"

#include <QCompleter>
#include <QGridLayout>
#include <QLabel>
#include <QPushButton>

#include <KIcon>
#include <KLocale>

using namespace Mailody;

RecipientWidget::RecipientWidget( QWidget* parent, QAbstractItemModel* model )
        : QWidget( parent ),
        m_lastState( To )
{
    QGridLayout* grid = new QGridLayout( this );
    grid->setSpacing( 6 );
    grid->setMargin( 6 );

    QLabel* tolabel = new QLabel( i18n( "Recipient:" )+' ', this );
    m_edit = new AddressLineEdit( this );
    tolabel->setBuddy( m_edit );
    m_edit->setFocus();

    QCompleter* completer = new QCompleter( this );
    completer->setCaseSensitivity( Qt::CaseInsensitive );
    completer->setCompletionColumn( 0 );
    completer->setCompletionMode( QCompleter::PopupCompletion );
    completer->setModel( model );
    completer->setCompletionRole( Qt::DisplayRole );

    // Don't use the setCompleter() function on the lineedit, because
    // on activation of a completion, it will fill the lineedit with the
    // value. We don't want that, we just want to add it.

    // Show the completion box under the lineedit, and show it for every change
    completer->setWidget( m_edit );
    connect( m_edit, SIGNAL( textChanged( const QString & ) ),
             completer, SLOT( setCompletionPrefix( const QString & ) ) );
    connect( m_edit, SIGNAL( textChanged( const QString & ) ),
             completer, SLOT( complete() ) );

    // On activation of an item, add it as receipient and mark as dirty.
    connect( completer, SIGNAL( activated( const QString& ) ),
             SLOT( slotAddAddress( const QString& ) ) );
    connect( completer, SIGNAL( activated( const QString& ) ),
             SIGNAL( dirty() ) );

    connect( m_edit,SIGNAL( shiftAddress() ), SLOT( slotShiftAddress() ) );
    connect( m_edit,SIGNAL( addAddress( const QString& ) ),
             SLOT( slotAddAddress( const QString& ) ) );
    connect( m_edit,SIGNAL( addAddress( const QString& ) ),
             SIGNAL( dirty() ) );
    connect( m_edit,SIGNAL( containsValidAddress( bool ) ),
             SLOT( slotContainsValidAddress( bool ) ) );

    m_add = new QPushButton( i18n( "Add" ), this );
    slotContainsValidAddress( false );
    connect( m_add, SIGNAL( clicked() ), SLOT( slotAddClicked() ) );
    connect( m_add, SIGNAL( clicked() ), SIGNAL( dirty() ) );

    m_addressbox = new ComposerRecipientWidget( this, 0, 5 );
    m_addressbox ->setContextMenuPolicy( Qt::CustomContextMenu );
    m_addressbox->setColumnCount( 5 );
    // since we don't want the pixmaps to be editable, we deal with it
    // ourselves
    m_addressbox->setEditTriggers( QAbstractItemView::NoEditTriggers );
    m_addressbox->setSelectionMode( QAbstractItemView::SingleSelection );
    m_addressbox->setSelectionBehavior( QAbstractItemView::SelectRows );
    m_addressbox->setSortingEnabled( false );

    QStringList headers;
    headers << i18n( "Address" ) << i18n( "To" ) << i18n( "Cc" ) << i18n( "Bcc" )
    << i18n( "Del" );
    m_addressbox->setHeaderLabels( headers );
    connect( m_addressbox, SIGNAL( itemClicked( QTreeWidgetItem*, int ) ),
             SLOT( slotEditAddress( QTreeWidgetItem*, int ) ) );
    connect( m_addressbox, SIGNAL( itemSelectionChanged() ), SLOT( slotUpdateLineEdit() ) );
    connect( m_addressbox,
             SIGNAL( customContextMenuRequested( const QPoint & ) ),
             SLOT( slotContextMenuAddressList() ) );

    grid->addWidget( tolabel,3,0, Qt::AlignRight );
    grid->addWidget( m_edit,3,1 );
    grid->addWidget( m_add,3,2 );
    grid->addWidget( m_addressbox,4,1,1,2 );
}

void RecipientWidget::slotAddAddress( const QString& text )
{
    if ( text.isEmpty() || text.contains( "@" ) != 1 )
        return;

    // Check for dups.
    for ( int i = 0; i < m_addressbox->topLevelItemCount() ; ++i ) {
        QTreeWidgetItem* item = m_addressbox->topLevelItem( i );
        if ( item->text( 0 ).indexOf( text ) != -1 ) {
            return;
        }
    }

    m_lastInserted = new QTreeWidgetItem( m_addressbox );
    m_lastInserted->setText( 0, text.trimmed() );

    // although we are in singleSelectionMode, setSelected will happily not
    // clear any previous selection ;-)
    m_addressbox->clearSelection();
    m_lastInserted->setSelected( true );
    m_lastInserted->setIcon( m_lastState, KIcon( "dialog-ok" ) );
    m_lastInserted->setIcon( 4, KIcon( "edit-delete" ) );
    m_addressbox->scrollToItem( m_lastInserted );
    m_lastInserted->setFlags( m_lastInserted->flags() | Qt::ItemIsEditable );
    m_edit->clear();
    m_edit->setFocus();
    slotUpdateLineEdit();
}

void RecipientWidget::slotContainsValidAddress( bool active )
{
    m_add->setEnabled( active );
}

void RecipientWidget::slotUpdateLineEdit()
{
    // we obiously do not want textChanged() emitted here, which would
    // remove all the text accept the P.
    m_edit->blockSignals( true );

    if ( m_addressbox->selectedItems().isEmpty() ) {
        m_edit->clear();
        m_edit->setHelp( false );
        m_edit->blockSignals( false );
        return;
    }
    QTreeWidgetItem* selectedItem = m_addressbox->selectedItems().first();
    if ( !selectedItem->icon( Cc ).isNull() )
        m_edit->setText( i18n( "Press enter to move the address to the bcc field" ) );
    else if ( !selectedItem->icon( Bcc ).isNull() )
        m_edit->setText( i18n( "Press enter to move the address to the to field" ) );
    else if ( !selectedItem->icon( To ).isNull() )
        m_edit->setText( i18n( "Press enter to move the address to the cc field" ) );

    m_edit->blockSignals( false );
    m_edit->setHelp( true );
}

void RecipientWidget::slotShiftAddress()
{
    QTreeWidgetItem* selectedItem = m_addressbox->selectedItems().first();
    if ( !selectedItem )
        return;
    if ( !selectedItem->icon( To ).isNull() )
        slotEditAddress( selectedItem, Cc );
    else if ( !selectedItem->icon( Cc ).isNull() )
        slotEditAddress( selectedItem, Bcc );
    else if ( !selectedItem->icon( Bcc ).isNull() )
        slotEditAddress( selectedItem, To );
    slotUpdateLineEdit();
}

void RecipientWidget::slotAddClicked()
{
    if ( m_edit->help() )
        slotShiftAddress();
    else
        slotAddAddress( m_edit->text() );
}

void RecipientWidget::slotEditAddress( QTreeWidgetItem* lvi, int c )
{
    if ( !lvi )
        return;

    if ( c == 0 ) {
        m_addressbox->editItem( lvi );
        return;
    }

    if ( c == 4 ) {
        delete lvi;
        return;
    }

    lvi->setIcon( To, QPixmap() );
    lvi->setIcon( Cc, QPixmap() );
    lvi->setIcon( Bcc, QPixmap() );
    lvi->setIcon( c, KIcon( "dialog-ok" ) );
    m_lastState=c;
}

void RecipientWidget::setRcpt( const QString& address,
                               TypeOfAddress addressType )
{
    m_lastState = addressType;
    slotAddAddress( address );
}

void RecipientWidget::slotSetAddress( const QString& address )
{
    m_edit->setText( address );
}

QStringList RecipientWidget::list( TypeOfAddress addressType ) const
{
    QStringList list;
    for ( int i = 0; i < m_addressbox->topLevelItemCount() ; ++i ) {
        QTreeWidgetItem* item = m_addressbox->topLevelItem( i );
        if ( !item->icon( addressType ).isNull() ) {
            list.append( item->text( 0 ).trimmed().toLatin1() );
        }
    }
    return list;
}

bool RecipientWidget::hasRecipients() const
{
    return m_addressbox->topLevelItemCount() >= 1;
}

#include "recipientwidget.moc"
