/*
   Copyright (C) 2006-2007 Omat Holding B.V. <info@omat.nl>
   Copyright (C) 2007 Frode M. Døving <frode@lnix.net>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
   MA 02110-1301,USA.
*/

#ifndef SENDMESSAGE_H
#define SENDMESSAGE_H

#include <mailtransport/transport.h>
#include <kmime/kmime_message.h>

#include <akonadi/collection.h>

#include <QObject>
#include <QStringList>
#include <KJob>

namespace Mailody
{

/**
 * @class SendMessage
 * This is the class which does the actual sending
 * shared between the composer and quickreply
 * @author Tom Albers <tomalbers@kde.nl>
 */
class SendMessage : public QObject
{
    Q_OBJECT

public:
    /**
     * Contructor
     * @param parent The parent widget
     */
    explicit SendMessage( QWidget* parent );

    /** Destructor */
    ~SendMessage();

    /** Enum to indicate if we are final or if this is a draft. */
    enum SendType { Draft, Final };

    /** Send the message
     * @param folder Save the send item in this folder
     * @param identity Use this identity
     * @param transportId Use this transport
     * @param plain The plain text of the message
     * @param html The rich text of the message
     */
    void send( const Akonadi::Collection& folder, int identity,
               const int transportId, const QString& plain, const QString& html );

    /** Compile the message to be send.
     * @param send The Send type.
     * @param identity The Identity to use for sending.
     * @param plain The plain text.
     * @param html The rich text.
     */
    KMime::Message* compileMessage( SendType send,
                                    int identity, const QString& plain, const QString& html );

    /** Replaces the recipient of the to-list
     * @param To The address */
    void setTos( const QStringList& To ) {
        m_tos = To;
    };

    /** Replaces the recipient of the cc-list
     * @param Cc The address */
    void setCcs( const QStringList& Cc ) {
        m_ccs = Cc;
    };

    /** Replaces the recipient of the bcc-list
     * @param Bcc The address */
    void setBccs( const QStringList& Bcc ) {
        m_bccs = Bcc;
    };

    /** Add a recipient to the to-list
     * @param To The address */
    void addTo( const QString& To ) {
        m_tos.append( To );
    };

    /** Add a recipient to the cc-list
     * @param Cc The address */
    void addCc( const QString& Cc ) {
        m_ccs.append( Cc );
    };

    /** Add a recipient to the bcc-list
     * @param Bcc The address */
    void addBcc( const QString& Bcc ) {
        m_bccs.append( Bcc );
    };

    /** Add an attachments to the message
     * @param Attachment in the form <URL, user visible name> */
    void addAttachments( const QHash<KUrl, QString> &Attachment ) {
        m_attachments = Attachment ;
    };

    /** Set the subject
     * @param Subject The subject of the message. */
    void setSubject( const QString& Subject ) {
        m_subject = Subject;
    };

    /** Set the in-reply-to header.
     * @param irt the text for the in reply to header. */
    void setIRT( const QString& irt ) {
        m_irt = irt;
    };

// @cond private
private:
    const char * encoding( const QString& data );
    QStringList m_tos, m_ccs, m_bccs;
    QHash<KUrl, QString> m_attachments;
    QString m_subject, m_irt;
    QWidget* m_parent;

private slots:
    void slotResult( KJob * job );
// @endcond

signals:
    /** Emitted when there is an error reported */
    void sendError( const QString& );

    /** Emitted when finished */
    void finished();
};

}

#endif
