/* This file is part of the KDE project
   Copyright (C) 2008 Omat Holding B.V. <info@omat.nl>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*/

#include "systemtray.h"

#include <QEvent>
#include <QFont>
#include <QFontMetrics>
#include <QPainter>

#include <KColorScheme>
#include <KDebug>
#include <KIcon>
#include <KIconEffect>
#include <KLocale>
#include <KNotification>
#include <KStatusNotifierItem>
#include <KGlobalSettings>

#include <akonadi/monitor.h>
#include <akonadi/itemfetchscope.h>
#include <kmime/kmime_message.h>
#include <kpimutils/email.h>


using namespace Mailody;

SystemTray::SystemTray( QObject *parent, QWidget *widget )
        : KStatusNotifierItem( parent ), m_widget( widget )
{
    setIconByName( "mailody" );
    setCategory( KStatusNotifierItem::Communications );

    Akonadi::ItemFetchScope scope;
    scope.fetchPayloadPart( "RFC822" );

    Akonadi::Monitor* monitor = new Akonadi::Monitor( this );
    monitor->setAllMonitored( true );
    monitor->setMimeTypeMonitored( "message/rfc822" );
    monitor->setItemFetchScope( scope );

    //setItemFetchScope( fetchScope );
    connect( monitor,
             SIGNAL( itemAdded( const Akonadi::Item, const Akonadi::Collection& ) ),
             SLOT( slotNewItem( const Akonadi::Item, const Akonadi::Collection& ) ) );
}

SystemTray::~SystemTray()
{
}

void SystemTray::slotNewItem( const Akonadi::Item item,
                              const Akonadi::Collection& collection )
{
    //kDebug() << item.id();

    if ( item.hasFlag( "\\Seen" ) ) {
        // kDebug() << "Message already seen";
        return;
    }

    if ( !item.hasPayload<KMime::Message::Ptr>() ) {
        // kDebug() << "No payload";
        return;
    }

    KMime::Message* msg = item.payload<KMime::Message::Ptr>().get();
    const QString from = msg->from()->asUnicodeString();
    QString email, name;
    KPIMUtils::extractEmailAddressAndName( from, email, name );
    const QString subject = msg->subject()->asUnicodeString();

    QString text = i18n( "New mail has arrived from %1.", name );
    if ( !subject.isEmpty() )
        text.append( i18n( "Subject: %1", subject ) );

    // kDebug() << from << subject;
    KNotification* notify = new KNotification( "newmail", m_widget );
    notify->setPixmap( QPixmap( "mailody" ) );
    notify->setText( text );
    notify->setTitle( i18n( "New email" ) );
    notify->setActions( QStringList( i18n( "Go to Mailody" ) ) );
    notify->addContext( "folder" , collection.name() ) ;
    connect( notify, SIGNAL( action1Activated() ),
             SIGNAL( raise() ) );
    notify->sendEvent();
}

bool SystemTray::event( QEvent * event )
{
    if ( event->type() == QEvent::ToolTip ) {
        // kDebug() << "Show new emails here...";
        return true;
    }
    return false;
}

void SystemTray::unreadCountChanged( int unread )
{
    // kDebug() << "Unread count is now " << unread;
    if ( unread == m_unread )
        return;

    m_unread=unread;

    if ( unread <= 0 ) {
        setIconByName( "mailody" );
    } else {
        // adapted from KMSystemTray::updateCount()
        int oldWidth = m_lightIconImage.size().width();

        if ( oldWidth == 0 )
            return;

        QString countStr = QString::number( unread );
        QFont f = KGlobalSettings::generalFont();
        f.setBold( true );

        float pointSize = f.pointSizeF();
        QFontMetrics fm( f );
        int w = fm.width( countStr );
        if ( w > ( oldWidth ) ) {
            pointSize *= float( oldWidth ) / float( w );
            f.setPointSizeF( pointSize );
        }

        // overlay
        QImage overlayImg = m_lightIconImage.copy();
        QPainter p( &overlayImg );
        p.setFont( f );
        KColorScheme scheme( QPalette::Active, KColorScheme::Window );
        p.setPen( scheme.foreground( KColorScheme::LinkText ).color() );
        p.drawText( overlayImg.rect(), Qt::AlignCenter, countStr );

        setOverlayIconByPixmap( QPixmap::fromImage( overlayImg ) );
    }
}

#include "systemtray.moc"
